@@ -1,23 +1,21 @@
-/* vim: set sw=2 ts=2 sts=2 et: */
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /*
- * ephy-download.c
- * This file is part of Epiphany
+ *  Copyright © 2011 Igalia S.L.
  *
- * Copyright © 2011 - Igalia S.L.
+ *  This file is part of Epiphany.
  *
- * Epiphany is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
  *
- * Epiphany is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ *  Epiphany is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
+ *  You should have received a copy of the GNU General Public License
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -43,6 +41,8 @@ struct _EphyDownload {
   char *destination;
   char *content_type;
 
+  gboolean show_notification;
+
   EphyDownloadActionType action;
   guint32 start_time;
   gboolean finished;
@@ -138,7 +138,7 @@ ephy_download_set_property (GObject      *object,
 const char *
 ephy_download_get_content_type (EphyDownload *download)
 {
-  g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), NULL);
+  g_assert (EPHY_IS_DOWNLOAD (download));
 
   return download->content_type;
 }
@@ -272,8 +272,8 @@ void
 ephy_download_set_destination_uri (EphyDownload *download,
                                    const char   *destination)
 {
-  g_return_if_fail (EPHY_IS_DOWNLOAD (download));
-  g_return_if_fail (destination != NULL);
+  g_assert (EPHY_IS_DOWNLOAD (download));
+  g_assert (destination != NULL);
 
   webkit_download_set_destination (download->download, destination);
   g_object_notify_by_pspec (G_OBJECT (download), obj_properties[PROP_DESTINATION]);
@@ -292,7 +292,7 @@ void
 ephy_download_set_action (EphyDownload          *download,
                           EphyDownloadActionType action)
 {
-  g_return_if_fail (EPHY_IS_DOWNLOAD (download));
+  g_assert (EPHY_IS_DOWNLOAD (download));
 
   download->action = action;
   g_object_notify_by_pspec (G_OBJECT (download), obj_properties[PROP_ACTION]);
@@ -309,7 +309,7 @@ ephy_download_set_action (EphyDownload          *download,
 WebKitDownload *
 ephy_download_get_webkit_download (EphyDownload *download)
 {
-  g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), NULL);
+  g_assert (EPHY_IS_DOWNLOAD (download));
 
   return download->download;
 }
@@ -325,7 +325,7 @@ ephy_download_get_webkit_download (EphyDownload *download)
 const char *
 ephy_download_get_destination_uri (EphyDownload *download)
 {
-  g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), NULL);
+  g_assert (EPHY_IS_DOWNLOAD (download));
 
   return webkit_download_get_destination (download->download);
 }
@@ -344,7 +344,7 @@ ephy_download_get_destination_uri (EphyDownload *download)
 EphyDownloadActionType
 ephy_download_get_action (EphyDownload *download)
 {
-  g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), EPHY_DOWNLOAD_ACTION_NONE);
+  g_assert (EPHY_IS_DOWNLOAD (download));
 
   return download->action;
 }
@@ -361,7 +361,7 @@ ephy_download_get_action (EphyDownload *download)
 guint32
 ephy_download_get_start_time (EphyDownload *download)
 {
-  g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), 0);
+  g_assert (EPHY_IS_DOWNLOAD (download));
 
   return download->start_time;
 }
@@ -375,7 +375,7 @@ ephy_download_get_start_time (EphyDownload *download)
 void
 ephy_download_cancel (EphyDownload *download)
 {
-  g_return_if_fail (EPHY_IS_DOWNLOAD (download));
+  g_assert (EPHY_IS_DOWNLOAD (download));
 
   webkit_download_cancel (download->download);
 }
@@ -383,7 +383,7 @@ ephy_download_cancel (EphyDownload *download)
 gboolean
 ephy_download_is_active (EphyDownload *download)
 {
-  g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), FALSE);
+  g_assert (EPHY_IS_DOWNLOAD (download));
 
   return !download->finished;
 }
@@ -391,7 +391,7 @@ ephy_download_is_active (EphyDownload *download)
 gboolean
 ephy_download_succeeded (EphyDownload *download)
 {
-  g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), FALSE);
+  g_assert (EPHY_IS_DOWNLOAD (download));
 
   return download->finished && !download->error;
 }
@@ -400,7 +400,7 @@ gboolean
 ephy_download_failed (EphyDownload *download,
                       GError      **error)
 {
-  g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), FALSE);
+  g_assert (EPHY_IS_DOWNLOAD (download));
 
   if (download->finished && download->error) {
     if (error)
@@ -451,6 +451,10 @@ ephy_download_do_download_action (EphyDownload          *download,
       LOG ("ephy_download_do_download_action: none");
       ret = TRUE;
       break;
+    case EPHY_DOWNLOAD_ACTION_DO_NOT_AUTO_OPEN:
+      LOG ("ephy_download_do_download_action: do_not_auto_open");
+      ret = TRUE;
+      break;
     default:
       g_assert_not_reached ();
   }
@@ -604,6 +608,8 @@ ephy_download_init (EphyDownload *download)
   download->action = EPHY_DOWNLOAD_ACTION_NONE;
 
   download->start_time = gtk_get_current_event_time ();
+
+  download->show_notification = TRUE;
 }
 
 static void
@@ -684,18 +690,52 @@ download_created_destination_cb (WebKitDownload *wk_download,
   g_free (content_type);
 }
 
+static void
+display_download_finished_notification (WebKitDownload *download)
+{
+  GApplication *application;
+  GtkWindow *toplevel;
+  const char *dest;
+
+  application = G_APPLICATION (ephy_embed_shell_get_default ());
+  toplevel = gtk_application_get_active_window (GTK_APPLICATION (application));
+  dest = webkit_download_get_destination (download);
+
+  if (!gtk_window_is_active (toplevel) && dest != NULL) {
+    char *filename;
+    char *message;
+    GNotification *notification;
+
+    filename = g_filename_display_basename (dest);
+    /* Translators: a desktop notification when a download finishes. */
+    message = g_strdup_printf (_("Finished downloading %s"), filename);
+    /* Translators: the title of the notification. */
+    notification = g_notification_new (_("Download finished"));
+    g_notification_set_body (notification, message);
+    g_application_send_notification (application, "download-finished", notification);
+
+    g_free (filename);
+    g_free (message);
+    g_object_unref (notification);
+  }
+}
+
 static void
 download_finished_cb (WebKitDownload *wk_download,
                       EphyDownload   *download)
 {
   download->finished = TRUE;
-  g_signal_emit (download, signals[COMPLETED], 0);
 
   if (g_settings_get_boolean (EPHY_SETTINGS_MAIN, EPHY_PREFS_AUTO_DOWNLOADS) &&
       download->action == EPHY_DOWNLOAD_ACTION_NONE)
     ephy_download_do_download_action (download, EPHY_DOWNLOAD_ACTION_OPEN, download->start_time);
   else
     ephy_download_do_download_action (download, download->action, download->start_time);
+
+  if (download->show_notification)
+    display_download_finished_notification (wk_download);
+
+  g_signal_emit (download, signals[COMPLETED], 0);
 }
 
 static void
@@ -724,7 +764,7 @@ ephy_download_new (WebKitDownload *download)
 {
   EphyDownload *ephy_download;
 
-  g_return_val_if_fail (WEBKIT_IS_DOWNLOAD (download), NULL);
+  g_assert (WEBKIT_IS_DOWNLOAD (download));
 
   ephy_download = g_object_new (EPHY_TYPE_DOWNLOAD, NULL);
 
@@ -765,7 +805,7 @@ ephy_download_new_for_uri (const char *uri)
   WebKitDownload *download;
   EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 
-  g_return_val_if_fail (uri != NULL, NULL);
+  g_assert (uri != NULL);
 
   download = webkit_web_context_download_uri (ephy_embed_shell_get_web_context (shell), uri);
   ephy_download = ephy_download_new (download);
@@ -773,3 +813,11 @@ ephy_download_new_for_uri (const char *uri)
 
   return ephy_download;
 }
+
+void
+ephy_download_disable_desktop_notification (EphyDownload *download)
+{
+  g_assert (EPHY_IS_DOWNLOAD (download));
+
+  download->show_notification = FALSE;
+}
