@@ -2,18 +2,20 @@
 /*
  *  Copyright © 2007 Xan Lopez
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  This file is part of Epiphany.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -57,8 +59,8 @@ ephy_embed_container_add_child (EphyEmbedContainer *container,
 {
   EphyEmbedContainerInterface *iface;
 
-  g_return_val_if_fail (EPHY_IS_EMBED_CONTAINER (container), -1);
-  g_return_val_if_fail (EPHY_IS_EMBED (child), -1);
+  g_assert (EPHY_IS_EMBED_CONTAINER (container));
+  g_assert (EPHY_IS_EMBED (child));
 
   iface = EPHY_EMBED_CONTAINER_GET_IFACE (container);
   return iface->add_child (container, child, position, set_active);
@@ -77,8 +79,8 @@ ephy_embed_container_set_active_child (EphyEmbedContainer *container,
 {
   EphyEmbedContainerInterface *iface;
 
-  g_return_if_fail (EPHY_IS_EMBED_CONTAINER (container));
-  g_return_if_fail (EPHY_IS_EMBED (child));
+  g_assert (EPHY_IS_EMBED_CONTAINER (container));
+  g_assert (EPHY_IS_EMBED (child));
 
   iface = EPHY_EMBED_CONTAINER_GET_IFACE (container);
 
@@ -98,8 +100,8 @@ ephy_embed_container_remove_child (EphyEmbedContainer *container,
 {
   EphyEmbedContainerInterface *iface;
 
-  g_return_if_fail (EPHY_IS_EMBED_CONTAINER (container));
-  g_return_if_fail (EPHY_IS_EMBED (child));
+  g_assert (EPHY_IS_EMBED_CONTAINER (container));
+  g_assert (EPHY_IS_EMBED (child));
 
   iface = EPHY_EMBED_CONTAINER_GET_IFACE (container);
 
@@ -119,7 +121,7 @@ ephy_embed_container_get_active_child (EphyEmbedContainer *container)
 {
   EphyEmbedContainerInterface *iface;
 
-  g_return_val_if_fail (EPHY_IS_EMBED_CONTAINER (container), NULL);
+  g_assert (EPHY_IS_EMBED_CONTAINER (container));
 
   iface = EPHY_EMBED_CONTAINER_GET_IFACE (container);
   return iface->get_active_child (container);
@@ -139,7 +141,7 @@ ephy_embed_container_get_children (EphyEmbedContainer *container)
 {
   EphyEmbedContainerInterface *iface;
 
-  g_return_val_if_fail (EPHY_IS_EMBED_CONTAINER (container), NULL);
+  g_assert (EPHY_IS_EMBED_CONTAINER (container));
 
   iface = EPHY_EMBED_CONTAINER_GET_IFACE (container);
   return iface->get_children (container);
@@ -158,10 +160,8 @@ ephy_embed_container_get_is_popup (EphyEmbedContainer *container)
 {
   EphyEmbedContainerInterface *iface;
 
-  g_return_val_if_fail (EPHY_IS_EMBED_CONTAINER (container), FALSE);
+  g_assert (EPHY_IS_EMBED_CONTAINER (container));
 
   iface = EPHY_EMBED_CONTAINER_GET_IFACE (container);
   return iface->get_is_popup (container);
 }
-
-
