@@ -1,19 +1,21 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2; -*- */
-/* vim: set sw=2 ts=2 sts=2 et: */
-/*  Copyright © 2008 Xan Lopez <xan@gnome.org>
+/*
+ *  Copyright © 2008 Xan Lopez <xan@gnome.org>
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This file is part of Epiphany.
+ *
+ *  Epiphany is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -25,10 +27,7 @@
 #include "ephy-langs.h"
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
-
-#include <glib.h>
-#include <math.h>
-#include <webkit2/webkit2.h>
+#include "ephy-user-agent.h"
 
 typedef struct {
   const char *schema;
@@ -37,7 +36,6 @@ typedef struct {
   void (*callback)(GSettings *settings, const char *key, gpointer data);
 } PrefData;
 
-#define ENABLE_SCRIPTS_SETTING "enable-javascript"
 #define DEFAULT_ENCODING_SETTING "default-charset"
 static WebKitSettings *webkit_settings = NULL;
 
@@ -110,131 +108,22 @@ webkit_pref_callback_user_stylesheet (GSettings  *settings,
   }
 }
 
-static char *
-webkit_pref_get_vendor_user_agent (void)
-{
-  char *vendor_user_agent = NULL;
-  GKeyFile *branding_keyfile;
-
-  branding_keyfile = g_key_file_new ();
-
-  if (g_key_file_load_from_file (branding_keyfile, SHARE_DIR "/branding.conf",
-                                 G_KEY_FILE_NONE, NULL)) {
-    char *vendor;
-    char *vendor_sub;
-    char *vendor_comment;
-
-    vendor = g_key_file_get_string (branding_keyfile,
-                                    "User Agent", "Vendor", NULL);
-    vendor_sub = g_key_file_get_string (branding_keyfile,
-                                        "User Agent", "VendorSub", NULL);
-    vendor_comment = g_key_file_get_string (branding_keyfile,
-                                            "User Agent", "VendorComment", NULL);
-
-    if (vendor) {
-      vendor_user_agent = g_strconcat (vendor,
-                                       vendor_sub ? "/" : "",
-                                       vendor_sub ? vendor_sub : "",
-                                       vendor_comment ? " (" : "",
-                                       vendor_comment ? vendor_comment : "",
-                                       vendor_comment ? ")" : "",
-                                       NULL);
-    }
-
-    g_free (vendor);
-    g_free (vendor_sub);
-    g_free (vendor_comment);
-  }
-
-  g_key_file_free (branding_keyfile);
-
-  return vendor_user_agent;
-}
-
-static const char *
-webkit_pref_get_internal_user_agent (void)
-{
-  static char *user_agent = NULL;
-  static gboolean initialized = FALSE;
-  char *vendor_user_agent;
-  const char *webkit_user_agent;
-
-  if (initialized)
-    return user_agent;
-
-  initialized = TRUE;
-
-  vendor_user_agent = webkit_pref_get_vendor_user_agent ();
-  webkit_user_agent = webkit_settings_get_user_agent (webkit_settings);
-
-  if (vendor_user_agent) {
-    user_agent = g_strdup_printf ("%s %s Epiphany/%s", webkit_user_agent,
-                                  vendor_user_agent, VERSION);
-    g_free (vendor_user_agent);
-  } else {
-    user_agent = g_strdup_printf ("%s Epiphany/%s", webkit_user_agent, VERSION);
-  }
-
-  return user_agent;
-}
-
 static void
 webkit_pref_callback_user_agent (GSettings  *settings,
                                  const char *key,
                                  gpointer    data)
 {
   EphyEmbedShell *shell = ephy_embed_shell_get_default ();
-  char *value;
   char *user_agent;
-  const char *base_user_agent;
-
-  value = g_settings_get_string (settings, key);
-  if (value != NULL && value[0] != '\0')
-    base_user_agent = value;
-  else
-    base_user_agent = webkit_pref_get_internal_user_agent ();
 
   if (ephy_embed_shell_get_mode (shell) == EPHY_EMBED_SHELL_MODE_APPLICATION)
-    user_agent = g_strdup_printf ("%s (Web App)", base_user_agent);
+    user_agent = g_strdup_printf ("%s (Web App)", ephy_user_agent_get_internal ());
   else
-    user_agent = g_strdup (base_user_agent);
+    user_agent = g_strdup (ephy_user_agent_get_internal ());
 
   webkit_settings_set_user_agent (webkit_settings, user_agent);
 
   g_free (user_agent);
-  g_free (value);
-}
-
-static gdouble
-get_screen_dpi (GdkScreen *screen)
-{
-  gdouble dpi;
-  gdouble dp, di;
-
-  dpi = gdk_screen_get_resolution (screen);
-  if (dpi != -1)
-    return dpi;
-
-  dp = hypot (gdk_screen_get_width (screen), gdk_screen_get_height (screen));
-  di = hypot (gdk_screen_get_width_mm (screen), gdk_screen_get_height_mm (screen)) / 25.4;
-
-  return dp / di;
-}
-
-static guint
-normalize_font_size (gdouble font_size)
-{
-  /* WebKit2 uses font sizes in pixels. */
-  GdkScreen *screen;
-  gdouble dpi;
-
-  /* FIXME: We should use the view screen instead of the detault one
-   * but we don't have access to the view here.
-   */
-  screen = gdk_screen_get_default ();
-  dpi = screen ? get_screen_dpi (screen) : 96;
-
-  return font_size / 72.0 * dpi;
 }
 
 static void
@@ -270,7 +159,7 @@ webkit_pref_callback_font_size (GSettings  *settings,
     pango_font_description_free (desc);
   }
 
-  g_object_set (webkit_settings, webkit_pref, normalize_font_size (size), NULL);
+  g_object_set (webkit_settings, webkit_pref, webkit_settings_font_size_to_pixels (size), NULL);
   g_free (value);
 }
 
@@ -318,7 +207,7 @@ normalize_languages (char **languages)
   langs = g_ptr_array_new ();
 
   for (i = 0; languages && languages[i]; i++) {
-    if (g_str_equal (languages[i], "system")) {
+    if (!strcmp (languages[i], "system")) {
       char **sys_langs = ephy_langs_get_languages ();
       int j;
 
@@ -360,10 +249,15 @@ webkit_pref_callback_accept_languages (GSettings  *settings,
       g_array_append_val (array, str);
     }
   }
+  g_strfreev (languages);
 
   ephy_langs_sanitise (array);
 
   webkit_web_context_set_preferred_languages (web_context, (const char * const *)(void *)array->data);
+  /* Used by the Firefox Sync web view in prefs-dialog.c. */
+  g_object_set_data_full (G_OBJECT (web_context), "preferred-languages",
+                          g_strdupv ((char **)(void *)array->data),
+                          (GDestroyNotify)g_strfreev);
 
   if (g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_ENABLE_SPELL_CHECKING)) {
     char **normalized = normalize_languages ((char **)(void *)array->data);
@@ -371,7 +265,6 @@ webkit_pref_callback_accept_languages (GSettings  *settings,
     g_strfreev (normalized);
   }
 
-  g_strfreev (languages);
   for (i = 0; i < array->len; i++)
     g_free (g_array_index (array, char *, i));
   g_array_free (array, TRUE);
@@ -384,11 +277,11 @@ ephy_embed_prefs_set_cookie_accept_policy (WebKitCookieManager *cookie_manager,
 {
   WebKitCookieAcceptPolicy policy;
 
-  if (g_str_equal (settings_policy, "never"))
+  if (!strcmp (settings_policy, "never"))
     policy = WEBKIT_COOKIE_POLICY_ACCEPT_NEVER;
-  else if (g_str_equal (settings_policy, "always"))
+  else if (!strcmp (settings_policy, "always"))
     policy = WEBKIT_COOKIE_POLICY_ACCEPT_ALWAYS;
-  else if (g_str_equal (settings_policy, "no-third-party"))
+  else if (!strcmp (settings_policy, "no-third-party"))
     policy = WEBKIT_COOKIE_POLICY_ACCEPT_NO_THIRD_PARTY;
   else {
     g_warn_if_reached ();
@@ -418,15 +311,17 @@ webkit_pref_callback_cookie_accept_policy (GSettings  *settings,
 }
 
 static void
-ephy_embed_prefs_update_font_settings (GSettings *ephy_settings, const char *key)
+webkit_pref_callback_gnome_fonts (GSettings  *ephy_settings,
+                                  const char *key,
+                                  gpointer    data)
 {
   if (g_settings_get_boolean (ephy_settings, key)) {
     g_object_set (webkit_settings,
                   "default-font-family", "serif",
                   "sans-serif-font-family", "sans-serif",
                   "monospace-font-family", "monospace",
-                  "default-font-size", normalize_font_size (12),
-                  "default-monospace-font-size", normalize_font_size (10),
+                  "default-font-size", webkit_settings_font_size_to_pixels (12),
+                  "default-monospace-font-size", webkit_settings_font_size_to_pixels (10),
                   NULL);
   } else {
     /* Sync with Epiphany values */
@@ -446,14 +341,6 @@ ephy_embed_prefs_update_font_settings (GSettings *ephy_settings, const char *key
   }
 }
 
-static void
-webkit_pref_callback_gnome_fonts (GSettings  *ephy_settings,
-                                  const char *key,
-                                  gpointer    data)
-{
-  ephy_embed_prefs_update_font_settings (ephy_settings, key);
-}
-
 static void
 webkit_pref_callback_enable_spell_checking (GSettings  *settings,
                                             const char *key,
@@ -479,15 +366,6 @@ webkit_pref_callback_enable_spell_checking (GSettings  *settings,
   }
 }
 
-static void
-gtk_settings_xft_dpi_changed_cb (GtkSettings *gtk_settings,
-                                 GParamSpec  *pspec,
-                                 gpointer     data)
-{
-  GSettings *gsettings = ephy_settings_get (EPHY_PREFS_WEB_SCHEMA);
-  ephy_embed_prefs_update_font_settings (gsettings, EPHY_PREFS_WEB_USE_GNOME_FONTS);
-}
-
 static const PrefData webkit_pref_entries[] =
 {
   /* Epiphany font settings */
@@ -534,8 +412,8 @@ static const PrefData webkit_pref_entries[] =
     EPHY_PREFS_WEB_LANGUAGE,
     "accept-language",
     webkit_pref_callback_accept_languages },
-  { EPHY_PREFS_SCHEMA,
-    EPHY_PREFS_USER_AGENT,
+  { EPHY_PREFS_WEB_SCHEMA,
+    EPHY_PREFS_WEB_USER_AGENT,
     "user-agent",
     webkit_pref_callback_user_agent },
   { EPHY_PREFS_WEB_SCHEMA,
@@ -547,12 +425,11 @@ static const PrefData webkit_pref_entries[] =
 static gpointer
 ephy_embed_prefs_init (gpointer user_data)
 {
-  GtkSettings *gtk_settings;
   guint i;
 
   webkit_settings = webkit_settings_new_with_settings ("enable-developer-extras", TRUE,
                                                        "enable-fullscreen", TRUE,
-                                                       "enable-site-specific-quirks", TRUE,
+                                                       "enable-javascript", TRUE,
                                                        "enable-dns-prefetching", TRUE,
                                                        "javascript-can-open-windows-automatically", TRUE,
                                                        NULL);
@@ -574,20 +451,6 @@ ephy_embed_prefs_init (gpointer user_data)
     g_free (key);
   }
 
-  /* Connect to the "notify::gtk-xft-dpi" signal for GtkSettings, so that
-   * we can update the font size in real time if the screen's resolution
-   * for font handling changes (e.g. enabled "Large Text" a11y mode).
-   */
-  gtk_settings = gtk_settings_get_default ();
-  if (gtk_settings) {
-    g_signal_connect (gtk_settings, "notify::gtk-xft-dpi",
-                      G_CALLBACK (gtk_settings_xft_dpi_changed_cb), NULL);
-  }
-
-  g_settings_bind (EPHY_SETTINGS_WEB,
-                   EPHY_PREFS_WEB_ENABLE_JAVASCRIPT,
-                   webkit_settings, ENABLE_SCRIPTS_SETTING,
-                   G_SETTINGS_BIND_GET);
   g_settings_bind (EPHY_SETTINGS_MAIN,
                    EPHY_PREFS_ENABLE_CARET_BROWSING,
                    webkit_settings, "enable-caret-browsing",
@@ -612,10 +475,14 @@ ephy_embed_prefs_init (gpointer user_data)
                    EPHY_PREFS_WEB_ENABLE_WEBAUDIO,
                    webkit_settings, "enable-webaudio",
                    G_SETTINGS_BIND_GET);
-  g_settings_bind (EPHY_SETTINGS_MAIN,
-                   EPHY_PREFS_ENABLE_SMOOTH_SCROLLING,
+  g_settings_bind (EPHY_SETTINGS_WEB,
+                   EPHY_PREFS_WEB_ENABLE_SMOOTH_SCROLLING,
                    webkit_settings, "enable-smooth-scrolling",
                    G_SETTINGS_BIND_GET);
+  g_settings_bind (EPHY_SETTINGS_WEB,
+                   EPHY_PREFS_WEB_ENABLE_SITE_SPECIFIC_QUIRKS,
+                   webkit_settings, "enable-site-specific-quirks",
+                   G_SETTINGS_BIND_GET);
   return webkit_settings;
 }
 
