@@ -32,11 +32,13 @@
 #include "ephy-encodings.h"
 #include "ephy-file-helpers.h"
 #include "ephy-filters-manager.h"
+#include "ephy-flatpak-utils.h"
 #include "ephy-history-service.h"
 #include "ephy-profile-utils.h"
 #include "ephy-settings.h"
 #include "ephy-snapshot-service.h"
 #include "ephy-tabs-catalog.h"
+#include "ephy-uri-helpers.h"
 #include "ephy-uri-tester-shared.h"
 #include "ephy-web-app-utils.h"
 #include "ephy-web-extension-proxy.h"
@@ -45,10 +47,6 @@
 #include <gtk/gtk.h>
 #include <stdlib.h>
 
-#if ENABLE_HTTPS_EVERYWHERE
-#include <httpseverywhere.h>
-#endif
-
 #define PAGE_SETUP_FILENAME "page-setup-gtk.ini"
 #define PRINT_SETTINGS_FILENAME "print-settings.ini"
 #define OVERVIEW_RELOAD_DELAY 500
@@ -56,6 +54,7 @@
 typedef struct {
   WebKitWebContext *web_context;
   EphyHistoryService *global_history_service;
+  EphyGSBService *global_gsb_service;
   EphyEncodings *encodings;
   GtkPageSetup *page_setup;
   GtkPrintSettings *print_settings;
@@ -71,6 +70,7 @@ typedef struct {
   EphyFiltersManager *filters_manager;
   EphySearchEngineManager *search_engine_manager;
   GCancellable *cancellable;
+  GList *app_origins;
 } EphyEmbedShellPrivate;
 
 enum {
@@ -78,6 +78,7 @@ enum {
   WEB_VIEW_CREATED,
   PAGE_CREATED,
   ALLOW_TLS_CERTIFICATE,
+  ALLOW_UNSAFE_BROWSING,
   FORM_AUTH_DATA_SAVE_REQUESTED,
   SENSITIVE_FORM_FOCUSED,
 
@@ -167,6 +168,7 @@ ephy_embed_shell_dispose (GObject *object)
   g_clear_object (&priv->page_setup);
   g_clear_object (&priv->print_settings);
   g_clear_object (&priv->global_history_service);
+  g_clear_object (&priv->global_gsb_service);
   g_clear_object (&priv->about_handler);
   g_clear_object (&priv->user_content);
   g_clear_object (&priv->downloads_manager);
@@ -179,6 +181,16 @@ ephy_embed_shell_dispose (GObject *object)
   G_OBJECT_CLASS (ephy_embed_shell_parent_class)->dispose (object);
 }
 
+static void
+ephy_embed_shell_finalize (GObject *object)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (EPHY_EMBED_SHELL (object));
+
+  g_list_free_full (priv->app_origins, g_free);
+
+  G_OBJECT_CLASS (ephy_embed_shell_parent_class)->dispose (object);
+}
+
 static void
 web_extension_form_auth_data_message_received_cb (WebKitUserContentManager *manager,
                                                   WebKitJavascriptResult   *message,
@@ -333,6 +345,17 @@ web_extension_tls_error_page_message_received_cb (WebKitUserContentManager *mana
   g_signal_emit (shell, signals[ALLOW_TLS_CERTIFICATE], 0, page_id);
 }
 
+static void
+web_extension_unsafe_browsing_error_page_message_received_cb (WebKitUserContentManager *manager,
+                                                              WebKitJavascriptResult   *message,
+                                                              EphyEmbedShell           *shell)
+{
+  guint64 page_id;
+
+  page_id = ephy_embed_utils_get_js_result_as_number (message);
+  g_signal_emit (shell, signals[ALLOW_UNSAFE_BROWSING], 0, page_id);
+}
+
 static void
 web_extension_about_apps_message_received_cb (WebKitUserContentManager *manager,
                                               WebKitJavascriptResult   *message,
@@ -460,7 +483,6 @@ delayed_thumbnail_update_cb (DelayedThumbnailUpdateData *data)
 void
 ephy_embed_shell_set_thumbnail_path (EphyEmbedShell *shell,
                                      const char     *url,
-                                     time_t          mtime,
                                      const char     *path)
 {
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
@@ -477,30 +499,24 @@ ephy_embed_shell_set_thumbnail_path (EphyEmbedShell *shell,
   }
 }
 
-typedef struct {
-  char *url;
-  time_t mtime;
-} GetSnapshotPathAsyncData;
-
 static void
-got_snapshot_path_for_url_cb (EphySnapshotService      *service,
-                              GAsyncResult             *result,
-                              GetSnapshotPathAsyncData *data)
+got_snapshot_path_for_url_cb (EphySnapshotService *service,
+                              GAsyncResult        *result,
+                              char                *url)
 {
   char *snapshot;
   GError *error = NULL;
 
   snapshot = ephy_snapshot_service_get_snapshot_path_for_url_finish (service, result, &error);
   if (snapshot) {
-    ephy_embed_shell_set_thumbnail_path (ephy_embed_shell_get_default (), data->url, data->mtime, snapshot);
+    ephy_embed_shell_set_thumbnail_path (ephy_embed_shell_get_default (), url, snapshot);
     g_free (snapshot);
   } else {
     /* Bad luck, not something to warn about. */
-    g_info ("Failed to get snapshot for URL %s: %s", data->url, error->message);
+    g_info ("Failed to get snapshot for URL %s: %s", url, error->message);
     g_error_free (error);
   }
-  g_free (data->url);
-  g_free (data);
+  g_free (url);
 }
 
 void
@@ -514,16 +530,13 @@ ephy_embed_shell_schedule_thumbnail_update (EphyEmbedShell *shell,
   snapshot = ephy_snapshot_service_lookup_cached_snapshot_path (service, url->url);
 
   if (snapshot) {
-    ephy_embed_shell_set_thumbnail_path (shell, url->url, url->thumbnail_time, snapshot);
+    ephy_embed_shell_set_thumbnail_path (shell, url->url, snapshot);
   } else {
-    GetSnapshotPathAsyncData *data = g_new (GetSnapshotPathAsyncData, 1);
-
-    data->url = g_strdup (url->url);
-    data->mtime = url->thumbnail_time;
     ephy_snapshot_service_get_snapshot_path_for_url_async (service,
-                                                           url->url, url->thumbnail_time, NULL,
+                                                           url->url,
+                                                           NULL,
                                                            (GAsyncReadyCallback)got_snapshot_path_for_url_cb,
-                                                           data);
+                                                           g_strdup (url->url));
   }
 }
 
@@ -538,7 +551,7 @@ ephy_embed_shell_get_global_history_service (EphyEmbedShell *shell)
 {
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
 
-  g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
+  g_assert (EPHY_IS_EMBED_SHELL (shell));
 
   if (priv->global_history_service == NULL) {
     char *filename;
@@ -553,7 +566,7 @@ ephy_embed_shell_get_global_history_service (EphyEmbedShell *shell)
     filename = g_build_filename (ephy_dot_dir (), EPHY_HISTORY_FILE, NULL);
     priv->global_history_service = ephy_history_service_new (filename, mode);
     g_free (filename);
-    g_return_val_if_fail (priv->global_history_service, NULL);
+    g_assert (priv->global_history_service);
     g_signal_connect (priv->global_history_service, "urls-visited",
                       G_CALLBACK (history_service_urls_visited_cb),
                       shell);
@@ -574,15 +587,35 @@ ephy_embed_shell_get_global_history_service (EphyEmbedShell *shell)
   return priv->global_history_service;
 }
 
-static void
-snapshot_saved_cb (EphySnapshotService *service,
-                   const char          *url,
-                   gint64               mtime,
-                   EphyEmbedShell      *shell)
+/**
+ * ephy_embed_shell_get_global_gsb_service:
+ * @shell: the #EphyEmbedShell
+ *
+ * Return value: (transfer none): the global #EphyGSBService
+ **/
+EphyGSBService *
+ephy_embed_shell_get_global_gsb_service (EphyEmbedShell *shell)
 {
-  ephy_history_service_set_url_thumbnail_time (ephy_embed_shell_get_global_history_service (shell),
-                                               url, mtime,
-                                               NULL, NULL, NULL);
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
+
+  if (priv->global_gsb_service == NULL) {
+    char *api_key;
+    char *dot_dir;
+    char *db_path;
+
+    api_key = g_settings_get_string (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_GSB_API_KEY);
+    dot_dir = ephy_default_dot_dir ();
+    db_path = g_build_filename (dot_dir, EPHY_GSB_FILE, NULL);
+    priv->global_gsb_service = ephy_gsb_service_new (api_key, db_path);
+
+    g_free (api_key);
+    g_free (dot_dir);
+    g_free (db_path);
+  }
+
+  return priv->global_gsb_service;
 }
 
 /**
@@ -596,7 +629,7 @@ ephy_embed_shell_get_encodings (EphyEmbedShell *shell)
 {
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
 
-  g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
+  g_assert (EPHY_IS_EMBED_SHELL (shell));
 
   if (priv->encodings == NULL)
     priv->encodings = ephy_encodings_new ();
@@ -876,26 +909,6 @@ ephy_embed_shell_create_web_context (EphyEmbedShell *shell)
   g_object_unref (manager);
 }
 
-#if ENABLE_HTTPS_EVERYWHERE
-static void
-https_everywhere_update_cb (HTTPSEverywhereUpdater *updater,
-                            GAsyncResult           *result)
-{
-  GError *error = NULL;
-
-  https_everywhere_updater_update_finish (updater, result, &error);
-
-  if (!error)
-    return;
-
-  if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) &&
-      !g_error_matches (error, HTTPS_EVERYWHERE_UPDATE_ERROR, HTTPS_EVERYWHERE_UPDATE_ERROR_IN_PROGRESS) &&
-      !g_error_matches (error, HTTPS_EVERYWHERE_UPDATE_ERROR, HTTPS_EVERYWHERE_UPDATE_ERROR_NO_UPDATE_AVAILABLE))
-    g_warning ("Failed to update HTTPS Everywhere rulesets: %s", error->message);
-  g_error_free (error);
-}
-#endif
-
 static char *
 adblock_filters_dir (EphyEmbedShell *shell)
 {
@@ -923,17 +936,9 @@ ephy_embed_shell_startup (GApplication *application)
   char *filename;
   char *cookie_policy;
   char *filters_dir;
-#if ENABLE_HTTPS_EVERYWHERE
-  HTTPSEverywhereContext *context;
-  HTTPSEverywhereUpdater *updater;
-#endif
 
   G_APPLICATION_CLASS (ephy_embed_shell_parent_class)->startup (application);
 
-  filters_dir = adblock_filters_dir (shell);
-  priv->filters_manager = ephy_filters_manager_new (filters_dir);
-  g_free (filters_dir);
-
   ephy_embed_shell_create_web_context (shell);
 
   ephy_embed_shell_setup_web_extensions_server (shell);
@@ -954,6 +959,12 @@ ephy_embed_shell_startup (GApplication *application)
                     G_CALLBACK (web_extension_tls_error_page_message_received_cb),
                     shell);
 
+  webkit_user_content_manager_register_script_message_handler (priv->user_content,
+                                                               "unsafeBrowsingErrorPage");
+  g_signal_connect (priv->user_content, "script-message-received::unsafeBrowsingErrorPage",
+                    G_CALLBACK (web_extension_unsafe_browsing_error_page_message_received_cb),
+                    shell);
+
   webkit_user_content_manager_register_script_message_handler (priv->user_content,
                                                                "formAuthData");
   g_signal_connect (priv->user_content, "script-message-received::formAuthData",
@@ -1029,23 +1040,9 @@ ephy_embed_shell_startup (GApplication *application)
   ephy_embed_prefs_set_cookie_accept_policy (cookie_manager, cookie_policy);
   g_free (cookie_policy);
 
-#if ENABLE_HTTPS_EVERYWHERE
-    /* We might want to be smarter about this in the future. For now,
-     * trigger an update of the rulesets once each time Epiphany is started.
-     * Note that the updated rules will not be used until the next time Epiphany
-     * is started. */
-  if (priv->mode != EPHY_EMBED_SHELL_MODE_TEST &&
-      priv->mode != EPHY_EMBED_SHELL_MODE_SEARCH_PROVIDER) {
-    context = https_everywhere_context_new ();
-    updater = https_everywhere_updater_new (context);
-    https_everywhere_updater_update (updater,
-                                     priv->cancellable,
-                                     (GAsyncReadyCallback)https_everywhere_update_cb,
-                                     NULL);
-    g_object_unref (context);
-    g_object_unref (updater);
-  }
-#endif
+  filters_dir = adblock_filters_dir (shell);
+  priv->filters_manager = ephy_filters_manager_new (filters_dir);
+  g_free (filters_dir);
 }
 
 static void
@@ -1123,10 +1120,6 @@ ephy_embed_shell_constructed (GObject *object)
     ephy_embed_shell_create_web_context (shell);
     priv->user_content = webkit_user_content_manager_new ();
   }
-
-  g_signal_connect_object (ephy_snapshot_service_get_default (),
-                           "snapshot-saved", G_CALLBACK (snapshot_saved_cb),
-                           shell, 0);
 }
 
 static void
@@ -1144,6 +1137,7 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
   GApplicationClass *application_class = G_APPLICATION_CLASS (klass);
 
   object_class->dispose = ephy_embed_shell_dispose;
+  object_class->finalize = ephy_embed_shell_finalize;
   object_class->set_property = ephy_embed_shell_set_property;
   object_class->get_property = ephy_embed_shell_get_property;
   object_class->constructed = ephy_embed_shell_constructed;
@@ -1229,6 +1223,22 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
                   G_TYPE_NONE, 1,
                   G_TYPE_UINT64);
 
+  /**
+   * EphyEmbedShell::allow-unsafe-browsing:
+   * @shell: the #EphyEmbedShell
+   * @page_id: the identifier of the web page
+   *
+   * Emitted when the web extension requests an exception be
+   * permitted for the unsafe browsing warning on the given page
+   */
+  signals[ALLOW_UNSAFE_BROWSING] =
+    g_signal_new ("allow-unsafe-browsing",
+                  EPHY_TYPE_EMBED_SHELL,
+                  G_SIGNAL_RUN_FIRST,
+                  0, NULL, NULL, NULL,
+                  G_TYPE_NONE, 1,
+                  G_TYPE_UINT64);
+
   /**
    * EphyEmbedShell::form-auth-data-save-requested:
    * @shell: the #EphyEmbedShell
@@ -1290,7 +1300,7 @@ ephy_embed_shell_set_page_setup (EphyEmbedShell *shell,
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   char *path;
 
-  g_return_if_fail (EPHY_IS_EMBED_SHELL (shell));
+  g_assert (EPHY_IS_EMBED_SHELL (shell));
 
   if (page_setup != NULL)
     g_object_ref (page_setup);
@@ -1317,7 +1327,7 @@ ephy_embed_shell_get_page_setup (EphyEmbedShell *shell)
 {
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
 
-  g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
+  g_assert (EPHY_IS_EMBED_SHELL (shell));
 
   if (priv->page_setup == NULL) {
     GError *error = NULL;
@@ -1353,7 +1363,7 @@ ephy_embed_shell_set_print_settings (EphyEmbedShell   *shell,
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   char *path;
 
-  g_return_if_fail (EPHY_IS_EMBED_SHELL (shell));
+  g_assert (EPHY_IS_EMBED_SHELL (shell));
 
   if (settings != NULL)
     g_object_ref (settings);
@@ -1381,7 +1391,7 @@ ephy_embed_shell_get_print_settings (EphyEmbedShell *shell)
 {
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
 
-  g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
+  g_assert (EPHY_IS_EMBED_SHELL (shell));
 
   if (priv->print_settings == NULL) {
     GError *error = NULL;
@@ -1413,11 +1423,56 @@ ephy_embed_shell_get_mode (EphyEmbedShell *shell)
 {
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
 
-  g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), EPHY_EMBED_SHELL_MODE_BROWSER);
+  g_assert (EPHY_IS_EMBED_SHELL (shell));
 
   return priv->mode;
 }
 
+/**
+ * ephy_embed_shell_add_app_related_uri:
+ * @shell: an #EphyEmbedShell
+ * @uri: the URI
+ **/
+void
+ephy_embed_shell_add_app_related_uri (EphyEmbedShell *shell, const char *uri)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+  char *origin;
+
+  g_assert (EPHY_IS_EMBED_SHELL (shell));
+  g_assert (priv->mode == EPHY_EMBED_SHELL_MODE_APPLICATION);
+
+  origin = ephy_uri_to_security_origin (uri);
+
+  if (!g_list_find_custom (priv->app_origins, origin, (GCompareFunc)g_strcmp0))
+    priv->app_origins = g_list_append (priv->app_origins, origin);
+}
+
+/**
+ * ephy_embed_shell_uri_looks_related_to_application:
+ * @shell: an #EphyEmbedShell
+ * @uri: the URI
+ *
+ * Returns: %TRUE if @uri looks related, %FALSE otherwise
+ **/
+gboolean
+ephy_embed_shell_uri_looks_related_to_app (EphyEmbedShell *shell,
+                                           const char     *uri)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  g_assert (EPHY_IS_EMBED_SHELL (shell));
+  g_assert (priv->mode == EPHY_EMBED_SHELL_MODE_APPLICATION);
+
+  for (GList *iter = priv->app_origins; iter != NULL; iter = iter->next) {
+    const char *iter_uri = (const char *)iter->data;
+    if (ephy_embed_utils_urls_have_same_origin (iter_uri, uri))
+      return TRUE;
+  }
+
+  return FALSE;
+}
+
 /**
  * ephy_embed_shell_launch_handler:
  * @shell: an #EphyEmbedShell
@@ -1441,8 +1496,12 @@ ephy_embed_shell_launch_handler (EphyEmbedShell *shell,
   GList *list = NULL;
   gboolean ret = FALSE;
 
-  g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), FALSE);
-  g_return_val_if_fail (file || mime_type, FALSE);
+  g_assert (EPHY_IS_EMBED_SHELL (shell));
+  g_assert (file || mime_type);
+
+  if (ephy_is_running_inside_flatpak ()) {
+    return ephy_file_launch_file_via_uri_handler (file);
+  }
 
   app = ephy_file_launcher_get_app_info_for_file (file, mime_type);
 
