@@ -2,18 +2,20 @@
 /*
  *  Copyright © 2014 Igalia S.L.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  This file is part of Epiphany.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -110,7 +112,7 @@ void
 ephy_web_overview_model_set_urls (EphyWebOverviewModel *model,
                                   GList                *urls)
 {
-  g_return_if_fail (EPHY_IS_WEB_OVERVIEW_MODEL (model));
+  g_assert (EPHY_IS_WEB_OVERVIEW_MODEL (model));
 
   g_list_free_full (model->items, (GDestroyNotify)ephy_web_overview_model_item_free);
   model->items = urls;
@@ -120,7 +122,7 @@ ephy_web_overview_model_set_urls (EphyWebOverviewModel *model,
 GList *
 ephy_web_overview_model_get_urls (EphyWebOverviewModel *model)
 {
-  g_return_val_if_fail (EPHY_IS_WEB_OVERVIEW_MODEL (model), NULL);
+  g_assert (EPHY_IS_WEB_OVERVIEW_MODEL (model));
 
   return model->items;
 }
@@ -132,7 +134,7 @@ ephy_web_overview_model_set_url_thumbnail (EphyWebOverviewModel *model,
 {
   const char *thumbnail_path;
 
-  g_return_if_fail (EPHY_IS_WEB_OVERVIEW_MODEL (model));
+  g_assert (EPHY_IS_WEB_OVERVIEW_MODEL (model));
 
   thumbnail_path = ephy_web_overview_model_get_url_thumbnail (model, url);
   if (g_strcmp0 (thumbnail_path, path) == 0)
@@ -146,7 +148,7 @@ const char *
 ephy_web_overview_model_get_url_thumbnail (EphyWebOverviewModel *model,
                                            const char           *url)
 {
-  g_return_val_if_fail (EPHY_IS_WEB_OVERVIEW_MODEL (model), NULL);
+  g_assert (EPHY_IS_WEB_OVERVIEW_MODEL (model));
 
   return g_hash_table_lookup (model->thumbnails, url);
 }
@@ -159,7 +161,7 @@ ephy_web_overview_model_set_url_title (EphyWebOverviewModel *model,
   GList *l;
   gboolean changed = FALSE;
 
-  g_return_if_fail (EPHY_IS_WEB_OVERVIEW_MODEL (model));
+  g_assert (EPHY_IS_WEB_OVERVIEW_MODEL (model));
 
   for (l = model->items; l; l = g_list_next (l)) {
     EphyWebOverviewModelItem *item = (EphyWebOverviewModelItem *)l->data;
@@ -186,7 +188,7 @@ ephy_web_overview_model_delete_url (EphyWebOverviewModel *model,
   GList *l;
   gboolean changed = FALSE;
 
-  g_return_if_fail (EPHY_IS_WEB_OVERVIEW_MODEL (model));
+  g_assert (EPHY_IS_WEB_OVERVIEW_MODEL (model));
 
   l = model->items;
   while (l) {
@@ -214,7 +216,7 @@ ephy_web_overview_model_delete_host (EphyWebOverviewModel *model,
   GList *l;
   gboolean changed = FALSE;
 
-  g_return_if_fail (EPHY_IS_WEB_OVERVIEW_MODEL (model));
+  g_assert (EPHY_IS_WEB_OVERVIEW_MODEL (model));
 
   l = model->items;
   while (l) {
@@ -240,7 +242,7 @@ ephy_web_overview_model_delete_host (EphyWebOverviewModel *model,
 void
 ephy_web_overview_model_clear (EphyWebOverviewModel *model)
 {
-  g_return_if_fail (EPHY_IS_WEB_OVERVIEW_MODEL (model));
+  g_assert (EPHY_IS_WEB_OVERVIEW_MODEL (model));
 
   if (!model->items)
     return;
