@@ -2,19 +2,20 @@
 /*
  *  Copyright © 2011 Igalia S.L.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  This file is part of Epiphany.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -249,19 +250,19 @@ static void ephy_smaps_pid_to_html (EphySMaps *smaps, GString *str, pid_t pid, E
       char *name = g_match_info_fetch (match_info, 1);
       char **size = NULL;
 
-      if (g_str_equal (name, "Size"))
+      if (!strcmp (name, "Size"))
         size = &vma->size;
-      else if (g_str_equal (name, "Rss"))
+      else if (!strcmp (name, "Rss"))
         size = &vma->rss;
-      else if (g_str_equal (name, "Pss"))
+      else if (!strcmp (name, "Pss"))
         size = &vma->pss;
-      else if (g_str_equal (name, "Shared_Clean"))
+      else if (!strcmp (name, "Shared_Clean"))
         size = &vma->shared_clean;
-      else if (g_str_equal (name, "Shared_Dirty"))
+      else if (!strcmp (name, "Shared_Dirty"))
         size = &vma->shared_dirty;
-      else if (g_str_equal (name, "Private_Clean"))
+      else if (!strcmp (name, "Private_Clean"))
         size = &vma->private_clean;
-      else if (g_str_equal (name, "Private_Dirty"))
+      else if (!strcmp (name, "Private_Dirty"))
         size = &vma->private_dirty;
 
       if (size)
@@ -420,7 +421,7 @@ static void ephy_smaps_pid_children_to_html (EphySMaps *smaps, GString *str, pid
     pid_t pid, ppid;
     EphyProcess process;
 
-    if (g_str_equal (name, "self"))
+    if (!strcmp (name, "self"))
       continue;
 
     pid = get_pid_from_proc_name (name);
@@ -487,5 +488,3 @@ EphySMaps *ephy_smaps_new (void)
 {
   return EPHY_SMAPS (g_object_new (EPHY_TYPE_SMAPS, NULL));
 }
-
-
