@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2; -*- */
-/* vim: set sw=2 ts=2 sts=2 et: */
 /*
  *  Copyright © 2011 Igalia S.L.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  This file is part of Epiphany.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -36,10 +37,11 @@ ephy_history_service_initialize_urls_table (EphyHistoryService *self)
                                   "host INTEGER NOT NULL REFERENCES hosts(id) ON DELETE CASCADE,"
                                   "url LONGVARCAR,"
                                   "title LONGVARCAR,"
+                                  "sync_id LONGVARCAR,"
                                   "visit_count INTEGER DEFAULT 0 NOT NULL,"
                                   "typed_count INTEGER DEFAULT 0 NOT NULL,"
                                   "last_visit_time INTEGER,"
-                                  "thumbnail_update_time INTEGER DEFAULT 0,"
+                                  "thumbnail_update_time INTEGER DEFAULT 0," /* this column is legacy, unused */
                                   "hidden_from_overview INTEGER DEFAULT 0)", &error);
 
   if (error) {
@@ -62,15 +64,15 @@ ephy_history_service_get_url_row (EphyHistoryService *self, const char *url_stri
   if (url_string == NULL && url != NULL)
     url_string = url->url;
 
-  g_return_val_if_fail (url_string || url->id != -1, NULL);
+  g_assert (url_string || url->id != -1);
 
   if (url != NULL && url->id != -1) {
     statement = ephy_sqlite_connection_create_statement (self->history_database,
-                                                         "SELECT id, url, title, visit_count, typed_count, last_visit_time, hidden_from_overview, thumbnail_update_time FROM urls "
+                                                         "SELECT id, url, title, visit_count, typed_count, last_visit_time, hidden_from_overview, sync_id FROM urls "
                                                          "WHERE id=?", &error);
   } else {
     statement = ephy_sqlite_connection_create_statement (self->history_database,
-                                                         "SELECT id, url, title, visit_count, typed_count, last_visit_time, hidden_from_overview, thumbnail_update_time FROM urls "
+                                                         "SELECT id, url, title, visit_count, typed_count, last_visit_time, hidden_from_overview, sync_id FROM urls "
                                                          "WHERE url=?", &error);
   }
 
@@ -112,9 +114,9 @@ ephy_history_service_get_url_row (EphyHistoryService *self, const char *url_stri
 
   url->visit_count = ephy_sqlite_statement_get_column_as_int (statement, 3),
   url->typed_count = ephy_sqlite_statement_get_column_as_int (statement, 4),
-  url->last_visit_time = ephy_sqlite_statement_get_column_as_int (statement, 5);
+  url->last_visit_time = ephy_sqlite_statement_get_column_as_int64 (statement, 5);
   url->hidden = ephy_sqlite_statement_get_column_as_int (statement, 6);
-  url->thumbnail_time = ephy_sqlite_statement_get_column_as_int (statement, 7);
+  url->sync_id = g_strdup (ephy_sqlite_statement_get_column_as_string (statement, 7));
 
   g_object_unref (statement);
   return url;
@@ -130,8 +132,8 @@ ephy_history_service_add_url_row (EphyHistoryService *self, EphyHistoryURL *url)
   g_assert (self->history_database != NULL);
 
   statement = ephy_sqlite_connection_create_statement (self->history_database,
-                                                       "INSERT INTO urls (url, title, visit_count, typed_count, last_visit_time, host) "
-                                                       " VALUES (?, ?, ?, ?, ?, ?)", &error);
+                                                       "INSERT INTO urls (url, title, visit_count, typed_count, last_visit_time, host, sync_id) "
+                                                       " VALUES (?, ?, ?, ?, ?, ?, ?)", &error);
   if (error) {
     g_warning ("Could not build urls table addition statement: %s", error->message);
     g_error_free (error);
@@ -142,8 +144,9 @@ ephy_history_service_add_url_row (EphyHistoryService *self, EphyHistoryURL *url)
       ephy_sqlite_statement_bind_string (statement, 1, url->title, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 2, url->visit_count, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 3, url->typed_count, &error) == FALSE ||
-      ephy_sqlite_statement_bind_int (statement, 4, url->last_visit_time, &error) == FALSE ||
-      ephy_sqlite_statement_bind_int (statement, 5, url->host->id, &error) == FALSE) {
+      ephy_sqlite_statement_bind_int64 (statement, 4, url->last_visit_time, &error) == FALSE ||
+      ephy_sqlite_statement_bind_int (statement, 5, url->host->id, &error) == FALSE ||
+      ephy_sqlite_statement_bind_string (statement, 6, url->sync_id, &error) == FALSE) {
     g_warning ("Could not insert URL into urls table: %s", error->message);
     g_error_free (error);
     g_object_unref (statement);
@@ -171,7 +174,7 @@ ephy_history_service_update_url_row (EphyHistoryService *self, EphyHistoryURL *u
   g_assert (self->history_database != NULL);
 
   statement = ephy_sqlite_connection_create_statement (self->history_database,
-                                                       "UPDATE urls SET title=?, visit_count=?, typed_count=?, last_visit_time=?, hidden_from_overview=?, thumbnail_update_time=? "
+                                                       "UPDATE urls SET title=?, visit_count=?, typed_count=?, last_visit_time=?, hidden_from_overview=?, sync_id=? "
                                                        "WHERE id=?", &error);
   if (error) {
     g_warning ("Could not build urls table modification statement: %s", error->message);
@@ -182,9 +185,9 @@ ephy_history_service_update_url_row (EphyHistoryService *self, EphyHistoryURL *u
   if (ephy_sqlite_statement_bind_string (statement, 0, url->title, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 1, url->visit_count, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 2, url->typed_count, &error) == FALSE ||
-      ephy_sqlite_statement_bind_int (statement, 3, url->last_visit_time, &error) == FALSE ||
+      ephy_sqlite_statement_bind_int64 (statement, 3, url->last_visit_time, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 4, url->hidden, &error) == FALSE ||
-      ephy_sqlite_statement_bind_int (statement, 5, url->thumbnail_time, &error) == FALSE ||
+      ephy_sqlite_statement_bind_string (statement, 5, url->sync_id, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 6, url->id, &error) == FALSE) {
     g_warning ("Could not modify URL in urls table: %s", error->message);
     g_error_free (error);
@@ -207,13 +210,13 @@ create_url_from_statement (EphySQLiteStatement *statement)
                                               ephy_sqlite_statement_get_column_as_string (statement, 2),
                                               ephy_sqlite_statement_get_column_as_int (statement, 3),
                                               ephy_sqlite_statement_get_column_as_int (statement, 4),
-                                              ephy_sqlite_statement_get_column_as_int (statement, 5));
+                                              ephy_sqlite_statement_get_column_as_int64 (statement, 5));
 
   url->id = ephy_sqlite_statement_get_column_as_int (statement, 0);
   url->host = ephy_history_host_new (NULL, NULL, 0, 1.0);
   url->hidden = ephy_sqlite_statement_get_column_as_int (statement, 6);
-  url->thumbnail_time = ephy_sqlite_statement_get_column_as_int (statement, 7);
-  url->host->id = ephy_sqlite_statement_get_column_as_int (statement, 8);
+  url->host->id = ephy_sqlite_statement_get_column_as_int (statement, 7);
+  url->sync_id = g_strdup (ephy_sqlite_statement_get_column_as_string (statement, 8));
 
   return url;
 }
@@ -235,8 +238,8 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
                                "urls.typed_count, "
                                "urls.last_visit_time, "
                                "urls.hidden_from_overview, "
-                               "urls.thumbnail_update_time, "
-                               "urls.host "
+                               "urls.host, "
+                               "urls.sync_id "
                                "FROM "
                                "urls ";
 
@@ -316,7 +319,7 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
   }
 
   if (query->from > 0) {
-    if (ephy_sqlite_statement_bind_int (statement, i++, (int)query->from, &error) == FALSE) {
+    if (ephy_sqlite_statement_bind_int64 (statement, i++, query->from, &error) == FALSE) {
       g_warning ("Could not build urls table query statement: %s", error->message);
       g_error_free (error);
       g_object_unref (statement);
@@ -324,7 +327,7 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
     }
   }
   if (query->to > 0) {
-    if (ephy_sqlite_statement_bind_int (statement, i++, (int)query->to, &error) == FALSE) {
+    if (ephy_sqlite_statement_bind_int64 (statement, i++, query->to, &error) == FALSE) {
       g_warning ("Could not build urls table query statement: %s", error->message);
       g_error_free (error);
       g_object_unref (statement);
@@ -393,7 +396,7 @@ ephy_history_service_delete_url (EphyHistoryService *self, EphyHistoryURL *url)
   g_assert (self->history_thread == g_thread_self ());
   g_assert (self->history_database != NULL);
 
-  g_return_if_fail (url->id != -1 || url->url);
+  g_assert (url->id != -1 || url->url);
 
   if (url->id != -1)
     sql_statement = "DELETE FROM urls WHERE id=?";
