@@ -1,32 +1,31 @@
-/* vim: set sw=2 ts=2 sts=2 et: */
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /*
- * ephy-download.c
- * This file is part of Epiphany
+ *  Copyright © 2011, 2015 Igalia S.L.
  *
- * Copyright © 2011, 2015 - Igalia S.L.
+ *  This file is part of Epiphany.
  *
- * Epiphany is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
  *
- * Epiphany is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ *  Epiphany is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
+ *  You should have received a copy of the GNU General Public License
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 #include "ephy-download-widget.h"
 
 #include "ephy-debug.h"
-#include "ephy-uri-helpers.h"
-#include "ephy-embed-shell.h"
 #include "ephy-downloads-manager.h"
+#include "ephy-embed-shell.h"
+#include "ephy-flatpak-utils.h"
+#include "ephy-uri-helpers.h"
 
 #include <glib/gi18n.h>
 #include <webkit2/webkit2.h>
@@ -276,7 +275,7 @@ widget_action_button_clicked_cb (EphyDownloadWidget *widget)
     ephy_downloads_manager_remove_download (manager, widget->download);
   } else {
     ephy_download_do_download_action (widget->download,
-                                      EPHY_DOWNLOAD_ACTION_BROWSE_TO,
+                                      ephy_is_running_inside_flatpak () ? EPHY_DOWNLOAD_ACTION_OPEN : EPHY_DOWNLOAD_ACTION_BROWSE_TO,
                                       gtk_get_current_event_time ());
   }
 }
@@ -488,7 +487,7 @@ ephy_download_widget_init (EphyDownloadWidget *widget)
 EphyDownload *
 ephy_download_widget_get_download (EphyDownloadWidget *widget)
 {
-  g_return_val_if_fail (EPHY_IS_DOWNLOAD_WIDGET (widget), NULL);
+  g_assert (EPHY_IS_DOWNLOAD_WIDGET (widget));
   return widget->download;
 }
 
@@ -506,7 +505,7 @@ ephy_download_widget_new (EphyDownload *ephy_download)
 {
   EphyDownloadWidget *widget;
 
-  g_return_val_if_fail (EPHY_IS_DOWNLOAD (ephy_download), NULL);
+  g_assert (EPHY_IS_DOWNLOAD (ephy_download));
 
   widget = g_object_new (EPHY_TYPE_DOWNLOAD_WIDGET,
                          "download", ephy_download, NULL);
