@@ -1,35 +1,39 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /*
  *  Copyright © 2003, 2004 Marco Pesenti Gritti
  *  Copyright © 2003, 2004 Christian Persch
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  This file is part of Epiphany.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 #include "ephy-location-controller.h"
 
-#include "ephy-widgets-type-builtins.h"
+
 #include "ephy-completion-model.h"
 #include "ephy-debug.h"
+#include "ephy-dnd.h"
 #include "ephy-embed-container.h"
 #include "ephy-embed-utils.h"
 #include "ephy-link.h"
-#include "ephy-dnd.h"
 #include "ephy-location-entry.h"
 #include "ephy-shell.h"
-#include "ephy-title-box.h"
+#include "ephy-title-widget.h"
+#include "ephy-uri-helpers.h"
+#include "ephy-widgets-type-builtins.h"
 
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
@@ -39,23 +43,20 @@
  * SECTION:ephy-location-controller
  * @short_description: An #EphyLink implementation
  *
- * #EphyLocationController handles navigation together with #EphyLocationEntry
+ * #EphyLocationController handles navigation together with an #EphyTitleWidget
  */
 
 struct _EphyLocationController {
   GObject parent_instance;
 
   EphyWindow *window;
-  EphyLocationEntry *location_entry;
-  EphyTitleBox *title_box;
-  GList *actions;
+  EphyTitleWidget *title_widget;
+  GtkGesture *longpress_gesture;
   char *address;
-  EphyNode *smart_bmks;
-  EphyBookmarks *bookmarks;
-  GdkPixbuf *icon;
-  guint editable : 1;
-  guint show_icon : 1;
+  gboolean editable;
   gboolean sync_address_is_blocked;
+  EphySearchEngineManager *search_engine_manager;
+  guint num_search_engines_actions;
 };
 
 static void ephy_location_controller_finalize (GObject *object);
@@ -69,21 +70,12 @@ enum {
   PROP_0,
   PROP_ADDRESS,
   PROP_EDITABLE,
-  PROP_ICON,
-  PROP_SHOW_ICON,
   PROP_WINDOW,
-  PROP_LOCATION_ENTRY,
-  PROP_TITLE_BOX,
+  PROP_TITLE_WIDGET,
   LAST_PROP
 };
 static GParamSpec *obj_properties[LAST_PROP];
 
-enum {
-  LOCK_CLICKED,
-  LAST_SIGNAL
-};
-static guint signals[LAST_SIGNAL];
-
 G_DEFINE_TYPE_WITH_CODE (EphyLocationController, ephy_location_controller, G_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (EPHY_TYPE_LINK,
                                                 NULL))
@@ -98,38 +90,6 @@ match_func (GtkEntryCompletion *completion,
   return TRUE;
 }
 
-static void
-action_activated_cb (GtkEntryCompletion     *completion,
-                     gint                    index,
-                     EphyLocationController *controller)
-{
-  GtkWidget *entry;
-  char *content;
-
-  entry = gtk_entry_completion_get_entry (completion);
-  content = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
-  if (content) {
-    EphyNode *node;
-    const char *smart_url;
-    char *url;
-
-    node = (EphyNode *)g_list_nth_data (controller->actions, index);
-    smart_url = ephy_node_get_property_string
-                  (node, EPHY_NODE_BMK_PROP_LOCATION);
-    g_return_if_fail (smart_url != NULL);
-
-    url = ephy_bookmarks_resolve_address
-            (controller->bookmarks, smart_url, content);
-    g_free (content);
-    if (url == NULL) return;
-
-    ephy_link_open (EPHY_LINK (controller), url, NULL,
-                    ephy_link_flags_from_current_event () | EPHY_LINK_TYPED);
-
-    g_free (url);
-  }
-}
-
 static void
 entry_drag_data_received_cb (GtkWidget *widget,
                              GdkDragContext *context,
@@ -183,7 +143,6 @@ static void
 entry_activate_cb (GtkEntry               *entry,
                    EphyLocationController *controller)
 {
-  EphyBookmarks *bookmarks;
   const char *content;
   char *address;
   char *effective_address;
@@ -194,13 +153,10 @@ entry_activate_cb (GtkEntry               *entry,
   }
 
   content = gtk_entry_get_text (entry);
-  if (content == NULL || content[0] == '\0') return;
-
-  bookmarks = ephy_shell_get_bookmarks (ephy_shell_get_default ());
-
-  address = ephy_bookmarks_resolve_address (bookmarks, content, NULL);
-  g_return_if_fail (address != NULL);
+  if (content == NULL || content[0] == '\0')
+    return;
 
+  address = g_strdup (content);
   effective_address = ephy_embed_utils_normalize_or_autosearch_address (g_strstrip (address));
   g_free (address);
 #if 0
@@ -245,7 +201,7 @@ user_changed_cb (GtkWidget *widget, EphyLocationController *controller)
   GtkTreeModel *model;
   GtkEntryCompletion *completion;
 
-  address = ephy_location_entry_get_location (EPHY_LOCATION_ENTRY (widget));
+  address = ephy_title_widget_get_address (EPHY_TITLE_WIDGET (widget));
 
   LOG ("user_changed_cb, address %s", address);
 
@@ -256,48 +212,29 @@ user_changed_cb (GtkWidget *widget, EphyLocationController *controller)
                                            update_done_cb, completion);
 }
 
-static void
-lock_clicked_cb (GtkWidget              *widget,
-                 EphyLocationController *controller)
-{
-  g_signal_emit (controller, signals[LOCK_CLICKED], 0);
-}
-
 static void
 sync_address (EphyLocationController *controller,
               GParamSpec             *pspec,
               GtkWidget              *widget)
 {
-  EphyLocationEntry *lentry = EPHY_LOCATION_ENTRY (widget);
-
   LOG ("sync_address %s", controller->address);
 
   g_signal_handlers_block_by_func (widget, G_CALLBACK (user_changed_cb), controller);
-  ephy_location_entry_set_location (lentry, controller->address);
-  ephy_title_box_set_address (controller->title_box, controller->address);
+  ephy_title_widget_set_address (controller->title_widget, controller->address);
   g_signal_handlers_unblock_by_func (widget, G_CALLBACK (user_changed_cb), controller);
 }
 
-static void
-title_box_mode_changed_cb (EphyTitleBox *title_box,
-                           GParamSpec   *psec,
-                           gpointer      user_data)
-{
-  EphyLocationController *controller = EPHY_LOCATION_CONTROLLER (user_data);
-
-  sync_address (controller, NULL, GTK_WIDGET (controller->location_entry));
-}
-
 static char *
 get_location_cb (EphyLocationEntry      *entry,
                  EphyLocationController *controller)
 {
   EphyEmbed *embed;
+  const char *address;
 
-  embed = ephy_embed_container_get_active_child
-            (EPHY_EMBED_CONTAINER (controller->window));
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (controller->window));
+  address = ephy_web_view_get_address (ephy_embed_get_web_view (embed));
 
-  return g_strdup (ephy_web_view_get_address (ephy_embed_get_web_view (embed)));
+  return ephy_embed_utils_is_no_show_address (address) ? NULL : ephy_uri_decode (address);
 }
 
 static char *
@@ -312,53 +249,17 @@ get_title_cb (EphyLocationEntry      *entry,
   return g_strdup (ephy_embed_get_title (embed));
 }
 
-static void
-remove_completion_actions (EphyLocationController *controller,
-                           EphyLocationEntry      *lentry)
-{
-  GtkEntryCompletion *completion;
-  GList *l;
-
-  completion = gtk_entry_get_completion (GTK_ENTRY (lentry));
-
-  for (l = controller->actions; l != NULL; l = l->next) {
-    gtk_entry_completion_delete_action (completion, 0);
-  }
-
-  g_signal_handlers_disconnect_by_func
-    (completion, G_CALLBACK (action_activated_cb), controller);
-}
-
-static void
-add_completion_actions (EphyLocationController *controller,
-                        EphyLocationEntry      *lentry)
-{
-  GtkEntryCompletion *completion;
-  GList *l;
-
-  completion = gtk_entry_get_completion (GTK_ENTRY (lentry));
-
-  for (l = controller->actions; l != NULL; l = l->next) {
-    EphyNode *bmk = l->data;
-    const char *title;
-    int index;
-
-    index = g_list_position (controller->actions, l);
-    title = ephy_node_get_property_string
-              (bmk, EPHY_NODE_BMK_PROP_TITLE);
-    gtk_entry_completion_insert_action_text (completion, index, (char *)title);
-  }
-
-  g_signal_connect (completion, "action_activated",
-                    G_CALLBACK (action_activated_cb), controller);
-}
-
 static gboolean
 focus_in_event_cb (GtkWidget              *entry,
                    GdkEventFocus          *event,
                    EphyLocationController *controller)
 {
-  if (!controller->sync_address_is_blocked) {
+  const char *address;
+
+  /* Never block sync if the location entry is empty, else homepage URL
+   * will be missing in homepage mode. */
+  address = ephy_title_widget_get_address (controller->title_widget);
+  if (!controller->sync_address_is_blocked && address && *address) {
     controller->sync_address_is_blocked = TRUE;
     g_signal_handlers_block_by_func (controller, G_CALLBACK (sync_address), entry);
   }
@@ -387,31 +288,125 @@ switch_page_cb (GtkNotebook            *notebook,
 {
   if (controller->sync_address_is_blocked == TRUE) {
     controller->sync_address_is_blocked = FALSE;
-    g_signal_handlers_unblock_by_func (controller, G_CALLBACK (sync_address), controller->location_entry);
+    g_signal_handlers_unblock_by_func (controller, G_CALLBACK (sync_address), controller->title_widget);
   }
 }
 
+static void
+action_activated_cb (GtkEntryCompletion     *completion,
+                     int                     index,
+                     EphyLocationController *controller)
+{
+  GtkWidget *entry;
+  char *content;
+  char *url;
+  char **engine_names;
+
+  entry = gtk_entry_completion_get_entry (completion);
+  content = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
+  if (content == NULL)
+    return;
+
+  engine_names = ephy_search_engine_manager_get_names (controller->search_engine_manager);
+  url = ephy_search_engine_manager_build_search_address (controller->search_engine_manager,
+                                                         engine_names[index],
+                                                         content);
+  g_strfreev (engine_names);
+
+  ephy_link_open (EPHY_LINK (controller), url, NULL,
+                  ephy_link_flags_from_current_event ());
+  g_free (content);
+  g_free (url);
+}
+
+static void
+fill_entry_completion_with_actions (GtkEntryCompletion     *completion,
+                                    EphyLocationController *controller)
+{
+  char **engine_names;
+
+  engine_names = ephy_search_engine_manager_get_names (controller->search_engine_manager);
+
+  controller->num_search_engines_actions = 0;
+
+  for (guint i = 0; engine_names[i] != NULL; i++) {
+    gtk_entry_completion_insert_action_text (completion, i, engine_names[i]);
+    controller->num_search_engines_actions++;
+  }
+
+  g_strfreev (engine_names);
+}
+
+static void
+add_completion_actions (EphyLocationController *controller,
+                        EphyLocationEntry      *lentry)
+{
+  GtkEntryCompletion *completion = gtk_entry_get_completion (GTK_ENTRY (lentry));
+
+  fill_entry_completion_with_actions (completion, controller);
+  g_signal_connect (completion, "action_activated",
+                    G_CALLBACK (action_activated_cb), controller);
+}
+
+static void
+search_engines_changed_cb (EphySearchEngineManager *manager,
+                           gpointer  data)
+{
+  EphyLocationController *controller;
+  GtkEntryCompletion *completion;
+
+  controller = EPHY_LOCATION_CONTROLLER (data);
+  completion = gtk_entry_get_completion (GTK_ENTRY (controller->title_widget));
+
+  for (guint i = 0; i < controller->num_search_engines_actions; i++)
+    gtk_entry_completion_delete_action (completion, 0);
+
+  fill_entry_completion_with_actions (completion, controller);
+}
+
+static void
+longpress_gesture_cb (GtkGestureLongPress *gesture,
+                      gdouble              x,
+                      gdouble              y,
+                      gpointer             user_data)
+{
+  EphyLocationController *controller = EPHY_LOCATION_CONTROLLER (user_data);
+
+  gtk_editable_select_region (GTK_EDITABLE (controller->title_widget), 0, -1);
+}
+
 static void
 ephy_location_controller_constructed (GObject *object)
 {
   EphyLocationController *controller = EPHY_LOCATION_CONTROLLER (object);
   EphyHistoryService *history_service;
-  EphyBookmarks *bookmarks;
+  EphyBookmarksManager *bookmarks_manager;
   EphyCompletionModel *model;
   GtkWidget *notebook, *widget;
 
   G_OBJECT_CLASS (ephy_location_controller_parent_class)->constructed (object);
 
   notebook = ephy_window_get_notebook (controller->window);
-  widget = GTK_WIDGET (controller->location_entry);
+  widget = GTK_WIDGET (controller->title_widget);
 
   g_signal_connect (notebook, "switch-page",
                     G_CALLBACK (switch_page_cb), controller);
 
-  history_service = EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (ephy_embed_shell_get_default ()));
-  bookmarks = ephy_shell_get_bookmarks (ephy_shell_get_default ());
-  model = ephy_completion_model_new (history_service, bookmarks);
-  ephy_location_entry_set_completion (controller->location_entry,
+  sync_address (controller, NULL, widget);
+  g_signal_connect_object (controller, "notify::address",
+                           G_CALLBACK (sync_address), widget, 0);
+
+  if (!EPHY_IS_LOCATION_ENTRY (controller->title_widget))
+    return;
+
+  controller->longpress_gesture = gtk_gesture_long_press_new (widget);
+  gtk_gesture_single_set_touch_only (GTK_GESTURE_SINGLE (controller->longpress_gesture), TRUE);
+  g_signal_connect (controller->longpress_gesture, "pressed", G_CALLBACK (longpress_gesture_cb), controller);
+
+  history_service = ephy_embed_shell_get_global_history_service (ephy_embed_shell_get_default ());
+  bookmarks_manager = ephy_shell_get_bookmarks_manager (ephy_shell_get_default ());
+  model = ephy_completion_model_new (history_service, bookmarks_manager);
+  ephy_location_entry_set_completion (EPHY_LOCATION_ENTRY (controller->title_widget),
                                       GTK_TREE_MODEL (model),
                                       EPHY_COMPLETION_TEXT_COL,
                                       EPHY_COMPLETION_ACTION_COL,
@@ -422,29 +417,18 @@ ephy_location_controller_constructed (GObject *object)
                                       EPHY_COMPLETION_FAVICON_COL);
   g_object_unref (model);
 
-  ephy_location_entry_set_match_func (controller->location_entry,
+  ephy_location_entry_set_match_func (EPHY_LOCATION_ENTRY (controller->title_widget),
                                       match_func,
-                                      controller->location_entry,
+                                      controller->title_widget,
                                       NULL);
 
-  add_completion_actions (controller, controller->location_entry);
+  add_completion_actions (controller, EPHY_LOCATION_ENTRY (controller->title_widget));
 
-  g_signal_connect_object (controller->title_box, "notify::mode",
-                           G_CALLBACK (title_box_mode_changed_cb), controller, 0);
+  g_signal_connect_object (controller->search_engine_manager, "changed",
+                           G_CALLBACK (search_engines_changed_cb), controller, 0);
 
-  sync_address (controller, NULL, widget);
-  g_signal_connect_object (controller, "notify::address",
-                           G_CALLBACK (sync_address), widget, 0);
   g_object_bind_property (controller, "editable",
-                          controller->location_entry, "editable",
-                          G_BINDING_SYNC_CREATE);
-
-  g_object_bind_property (controller, "icon",
-                          controller->location_entry, "favicon",
-                          G_BINDING_SYNC_CREATE);
-
-  g_object_bind_property (controller, "show-icon",
-                          controller->location_entry, "show-favicon",
+                          controller->title_widget, "editable",
                           G_BINDING_SYNC_CREATE);
 
   g_signal_connect_object (widget, "drag-data-received",
@@ -455,8 +439,6 @@ ephy_location_controller_constructed (GObject *object)
                            controller, 0);
   g_signal_connect_object (widget, "user-changed",
                            G_CALLBACK (user_changed_cb), controller, 0);
-  g_signal_connect_object (widget, "lock-clicked",
-                           G_CALLBACK (lock_clicked_cb), controller, 0);
   g_signal_connect_object (widget, "get-location",
                            G_CALLBACK (get_location_cb), controller, 0);
   g_signal_connect_object (widget, "get-title",
@@ -482,23 +464,11 @@ ephy_location_controller_set_property (GObject      *object,
     case PROP_EDITABLE:
       controller->editable = g_value_get_boolean (value);
       break;
-    case PROP_ICON:
-      if (controller->icon != NULL) {
-        g_object_unref (controller->icon);
-      }
-      controller->icon = GDK_PIXBUF (g_value_dup_object (value));
-      break;
-    case PROP_SHOW_ICON:
-      controller->show_icon = g_value_get_boolean (value);
-      break;
     case PROP_WINDOW:
       controller->window = EPHY_WINDOW (g_value_get_object (value));
       break;
-    case PROP_LOCATION_ENTRY:
-      controller->location_entry = EPHY_LOCATION_ENTRY (g_value_get_object (value));
-      break;
-    case PROP_TITLE_BOX:
-      controller->title_box = EPHY_TITLE_BOX (g_value_get_object (value));
+    case PROP_TITLE_WIDGET:
+      controller->title_widget = EPHY_TITLE_WIDGET (g_value_get_object (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -520,12 +490,6 @@ ephy_location_controller_get_property (GObject    *object,
     case PROP_EDITABLE:
       g_value_set_boolean (value, controller->editable);
       break;
-    case PROP_ICON:
-      g_value_set_object (value, controller->icon);
-      break;
-    case PROP_SHOW_ICON:
-      g_value_set_boolean (value, controller->show_icon);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -540,19 +504,21 @@ ephy_location_controller_dispose (GObject *object)
   notebook = ephy_window_get_notebook (controller->window);
 
   if (notebook == NULL ||
-      controller->location_entry == NULL) {
+      controller->title_widget == NULL) {
     return;
   }
 
-  g_signal_handlers_disconnect_matched (controller, G_SIGNAL_MATCH_DATA,
-                                        0, 0, NULL, NULL, controller->location_entry);
-  g_signal_handlers_disconnect_matched (controller->location_entry, G_SIGNAL_MATCH_DATA,
-                                        0, 0, NULL, NULL, controller);
-  g_signal_handlers_disconnect_matched (controller->title_box, G_SIGNAL_MATCH_DATA,
-                                        0, 0, NULL, NULL, controller);
+  g_clear_object (&controller->longpress_gesture);
+
+  if (EPHY_IS_LOCATION_ENTRY (controller->title_widget)) {
+    g_signal_handlers_disconnect_matched (controller, G_SIGNAL_MATCH_DATA,
+                                          0, 0, NULL, NULL, controller->title_widget);
+    g_signal_handlers_disconnect_matched (controller->title_widget, G_SIGNAL_MATCH_DATA,
+                                          0, 0, NULL, NULL, controller);
+  }
   g_signal_handlers_disconnect_matched (notebook, G_SIGNAL_MATCH_DATA,
                                         0, 0, NULL, NULL, controller);
-  controller->location_entry = NULL;
+  controller->title_widget = NULL;
 
   G_OBJECT_CLASS (ephy_location_controller_parent_class)->dispose (object);
 }
@@ -568,21 +534,6 @@ ephy_location_controller_class_init (EphyLocationControllerClass *class)
   object_class->get_property = ephy_location_controller_get_property;
   object_class->set_property = ephy_location_controller_set_property;
 
-  /**
-   * EphyLocationController::lock-clicked:
-   * @controller: the object which received the signal.
-   *
-   * Emitted when the user clicks on the security icon of the internal
-   * #EphyLocationEntry.
-   */
-  signals[LOCK_CLICKED] = g_signal_new (
-    "lock-clicked",
-    EPHY_TYPE_LOCATION_CONTROLLER,
-    G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
-    0, NULL, NULL, NULL,
-    G_TYPE_NONE,
-    0);
-
   /**
    * EphyLocationController:address:
    *
@@ -607,30 +558,6 @@ ephy_location_controller_class_init (EphyLocationControllerClass *class)
                           TRUE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
-  /**
-   * EphyLocationController:icon:
-   *
-   * The icon corresponding to the current location.
-   */
-  obj_properties[PROP_ICON] =
-    g_param_spec_object ("icon",
-                         "Icon",
-                         "The icon corresponding to the current location",
-                         GDK_TYPE_PIXBUF,
-                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-
-  /**
-   * EphyLocationController:show-icon:
-   *
-   * If we should show the page icon.
-   */
-  obj_properties[PROP_SHOW_ICON] =
-    g_param_spec_boolean ("show-icon",
-                          "Show Icon",
-                          "Whether to show the favicon",
-                          TRUE,
-                          G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-
   /**
    * EphyLocationController:window:
    *
@@ -644,143 +571,30 @@ ephy_location_controller_class_init (EphyLocationControllerClass *class)
                          G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT_ONLY);
 
   /**
-   * EphyLocationController:location-entry:
+   * EphyLocationController:title-widget:
    *
-   * The controlled location entry.
+   * The #EphyLocationController sets the address of the #EphyTitleWidget.
    */
-  obj_properties[PROP_LOCATION_ENTRY] =
-    g_param_spec_object ("location-entry",
-                         "Location entry",
-                         "The controlled location entry",
-                         G_TYPE_OBJECT,
-                         G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT_ONLY);
-
-  /**
-   * EphyLocationController:title-box:
-   *
-   * The #EphyLocationController sets the address of this title box.
-   */
-  obj_properties[PROP_TITLE_BOX] =
-    g_param_spec_object ("title-box",
-                         "Title box",
-                         "The title box whose address will be managed",
+  obj_properties[PROP_TITLE_WIDGET] =
+    g_param_spec_object ("title-widget",
+                         "Title widget",
+                         "The title widget whose address will be managed",
                          G_TYPE_OBJECT,
                          G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT_ONLY);
 
   g_object_class_install_properties (object_class, LAST_PROP, obj_properties);
 }
 
-static int
-compare_actions (gconstpointer a,
-                 gconstpointer b)
-{
-  EphyNode *node_a = (EphyNode *)a;
-  EphyNode *node_b = (EphyNode *)b;
-  const char *title1, *title2;
-  int retval;
-
-  title1 = ephy_node_get_property_string (node_a, EPHY_NODE_BMK_PROP_TITLE);
-  title2 = ephy_node_get_property_string (node_b, EPHY_NODE_BMK_PROP_TITLE);
-
-  if (title1 == NULL) {
-    retval = -1;
-  } else if (title2 == NULL) {
-    retval = 1;
-  } else {
-    char *str_a, *str_b;
-
-    str_a = g_utf8_casefold (title1, -1);
-    str_b = g_utf8_casefold (title2, -1);
-    retval = g_utf8_collate (str_a, str_b);
-    g_free (str_a);
-    g_free (str_b);
-  }
-
-  return retval;
-}
-
-static void
-init_actions_list (EphyLocationController *controller)
-{
-  GPtrArray *children;
-  guint i;
-
-  children = ephy_node_get_children (controller->smart_bmks);
-  for (i = 0; i < children->len; i++) {
-    EphyNode *kid;
-
-    kid = g_ptr_array_index (children, i);
-
-    controller->actions = g_list_prepend
-                            (controller->actions, kid);
-  }
-
-  controller->actions =
-    g_list_sort (controller->actions, (GCompareFunc)compare_actions);
-}
-
-static void
-update_actions_list (EphyLocationController *controller)
-{
-  remove_completion_actions (controller, controller->location_entry);
-
-  g_list_free (controller->actions);
-  controller->actions = NULL;
-  init_actions_list (controller);
-
-  add_completion_actions (controller, controller->location_entry);
-}
-
-static void
-actions_child_removed_cb (EphyNode               *node,
-                          EphyNode               *child,
-                          guint                   old_index,
-                          EphyLocationController *controller)
-{
-  update_actions_list (controller);
-}
-
-static void
-actions_child_added_cb (EphyNode               *node,
-                        EphyNode               *child,
-                        EphyLocationController *controller)
-{
-  update_actions_list (controller);
-}
-
-static void
-actions_child_changed_cb (EphyNode               *node,
-                          EphyNode               *child,
-                          guint                   property_id,
-                          EphyLocationController *controller)
-{
-  update_actions_list (controller);
-}
-
 static void
 ephy_location_controller_init (EphyLocationController *controller)
 {
+  EphyEmbedShell *shell;
+
   controller->address = g_strdup ("");
   controller->editable = TRUE;
-  controller->bookmarks = ephy_shell_get_bookmarks (ephy_shell_get_default ());
-  controller->smart_bmks = ephy_bookmarks_get_smart_bookmarks
-                             (controller->bookmarks);
   controller->sync_address_is_blocked = FALSE;
-
-  init_actions_list (controller);
-
-  ephy_node_signal_connect_object (controller->smart_bmks,
-                                   EPHY_NODE_CHILD_ADDED,
-                                   (EphyNodeCallback)actions_child_added_cb,
-                                   G_OBJECT (controller));
-  ephy_node_signal_connect_object (controller->smart_bmks,
-                                   EPHY_NODE_CHILD_REMOVED,
-                                   (EphyNodeCallback)actions_child_removed_cb,
-                                   G_OBJECT (controller));
-  ephy_node_signal_connect_object (controller->smart_bmks,
-                                   EPHY_NODE_CHILD_CHANGED,
-                                   (EphyNodeCallback)actions_child_changed_cb,
-                                   G_OBJECT (controller));
+  shell = ephy_embed_shell_get_default ();
+  controller->search_engine_manager = ephy_embed_shell_get_search_engine_manager (shell);
 }
 
 static void
@@ -788,11 +602,6 @@ ephy_location_controller_finalize (GObject *object)
 {
   EphyLocationController *controller = EPHY_LOCATION_CONTROLLER (object);
 
-  if (controller->icon != NULL) {
-    g_object_unref (controller->icon);
-  }
-
-  g_list_free (controller->actions);
   g_free (controller->address);
 
   G_OBJECT_CLASS (ephy_location_controller_parent_class)->finalize (object);
@@ -809,7 +618,7 @@ ephy_location_controller_finalize (GObject *object)
 const char *
 ephy_location_controller_get_address (EphyLocationController *controller)
 {
-  g_return_val_if_fail (EPHY_IS_LOCATION_CONTROLLER (controller), "");
+  g_assert (EPHY_IS_LOCATION_CONTROLLER (controller));
 
   return controller->address;
 }
@@ -825,7 +634,7 @@ void
 ephy_location_controller_set_address (EphyLocationController *controller,
                                       const char             *address)
 {
-  g_return_if_fail (EPHY_IS_LOCATION_CONTROLLER (controller));
+  g_assert (EPHY_IS_LOCATION_CONTROLLER (controller));
 
   LOG ("set_address %s", address);
 
