@@ -4,45 +4,44 @@
  *  Copyright © 2006, 2008 Christian Persch
  *  Copyright © 2011, 2012 Igalia S.L.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  This file is part of Epiphany.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
 #include "ephy-debug.h"
 #include "ephy-file-helpers.h"
-#include "ephy-initial-state.h"
-#include "ephy-private.h"
 #include "ephy-profile-utils.h"
 #include "ephy-session.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-string.h"
+#include "ephy-vcs-version.h"
 #include "ephy-web-app-utils.h"
 
 #include <errno.h>
 #include <glib/gi18n.h>
+#include <glib-unix.h>
 #include <gtk/gtk.h>
 #include <libnotify/notify.h>
+#include <libxml/xmlreader.h>
 #include <libxml/xmlversion.h>
+#include <signal.h>
 #include <string.h>
-
-#ifdef GDK_WINDOWING_X11
-#include <gdk/gdkx.h>
-#endif
+#include <stdlib.h>
 
 static gboolean open_in_new_tab = FALSE;
 static gboolean open_in_new_window = FALSE;
@@ -56,20 +55,58 @@ static char *application_to_delete = NULL;
 static gboolean private_instance = FALSE;
 static gboolean incognito_mode = FALSE;
 static gboolean application_mode = FALSE;
+static char *desktop_file_basename = NULL;
 static char *profile_directory = NULL;
 
+static EphyShell *ephy_shell = NULL;
+static int shutdown_signum = 0;
+
+static gboolean
+handle_shutdown_signal (gpointer user_data)
+{
+  shutdown_signum = GPOINTER_TO_INT (user_data);
+
+  /* Note that this function executes on the main loop AFTER the signal handler
+   * has returned, so we don't have to worry about async signal safety.
+   */
+  g_assert (ephy_shell != NULL);
+  ephy_shell_try_quit (ephy_shell);
+
+  /* Goals:
+   *
+   * (1) Shutdown safely and cleanly if signal is received once.
+   * (2) Shutdown unsafely but immediately if signal is received twice.
+   * (3) Always re-raise the signal so the parent process knows what happened.
+   *
+   * Removing this source is required by goals (2) and (3).
+   */
+  return G_SOURCE_REMOVE;
+}
+
+static gboolean
+application_mode_cb (const gchar *option_name,
+                     const gchar *value,
+                     gpointer     data,
+                     GError     **error)
+{
+  application_mode = TRUE;
+  desktop_file_basename = g_strdup (value);
+  return TRUE;
+}
+
 static gboolean
 option_version_cb (const gchar *option_name,
                    const gchar *value,
                    gpointer     data,
                    GError     **error)
 {
-  g_print ("%s %s\n", _("Web"), VERSION);
+  g_print ("%s %s\n", _("Web"), VCSVERSION);
 
   exit (EXIT_SUCCESS);
   return FALSE;
 }
- 
+
+/* If you're modifying this array then you need to update the manpage. */
 static const GOptionEntry option_entries[] =
 {
   { "new-tab", 'n', 0, G_OPTION_ARG_NONE, &open_in_new_tab,
@@ -86,15 +123,14 @@ static const GOptionEntry option_entries[] =
     N_("Start a private instance"), NULL },
   { "incognito-mode", 'i', 0, G_OPTION_ARG_NONE, &incognito_mode,
     N_("Start an instance in incognito mode"), NULL },
-  { "netbank-mode", 0, 0, G_OPTION_ARG_NONE, &incognito_mode,
-    N_("Start an instance in netbank mode"), NULL },
-  { "application-mode", 'a', 0, G_OPTION_ARG_NONE, &application_mode,
+  { "application-mode", 'a', G_OPTION_FLAG_FILENAME | G_OPTION_FLAG_OPTIONAL_ARG,
+    G_OPTION_ARG_CALLBACK, application_mode_cb,
     N_("Start the browser in application mode"), NULL },
   { "profile", 0, 0, G_OPTION_ARG_STRING, &profile_directory,
     N_("Profile directory to use in the private instance"), N_("DIR") },
   { G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &arguments,
-    "", N_("URL …")},
-  { "version", 0, G_OPTION_FLAG_NO_ARG | G_OPTION_FLAG_HIDDEN, 
+    "", N_("URL …") },
+  { "version", 0, G_OPTION_FLAG_NO_ARG | G_OPTION_FLAG_HIDDEN,
     G_OPTION_ARG_CALLBACK, option_version_cb, NULL, NULL },
   { "delete-application", 0, 0, G_OPTION_ARG_STRING | G_OPTION_FLAG_HIDDEN,
     &application_to_delete, NULL, NULL },
@@ -109,7 +145,8 @@ get_startup_id (void)
   guint32 retval = 0;
 
   startup_id = g_getenv ("DESKTOP_STARTUP_ID");
-  if (startup_id == NULL) return 0;
+  if (startup_id == NULL)
+    return 0;
 
   /* Find the launch time from the startup_id, if it's there.  Newer spec
    * states that the startup_id is of the form <unique>_TIME<timestamp>
@@ -119,96 +156,18 @@ get_startup_id (void)
     gulong value;
     gchar *end;
     errno = 0;
-    
+
     /* Skip past the "_TIME" part */
     time_str += 5;
-    
+
     value = strtoul (time_str, &end, 0);
     if (end != time_str && errno == 0)
-      retval = (guint32) value;
+      retval = (guint32)value;
   }
 
   return retval;
 }
 
-#ifdef GDK_WINDOWING_X11
-/*
- * FIXME: Need a solution for windowing-systems other than X11.
- */
-
-/* Copied from libnautilus/nautilus-program-choosing.c; Needed in case
- * we have no DESKTOP_STARTUP_ID (with its accompanying timestamp).
- */
-static Time
-slowly_and_stupidly_obtain_timestamp (Display *xdisplay)
-{
-  Window xwindow;
-  XEvent event;
-  
-  {
-    XSetWindowAttributes attrs;
-    Atom atom_name;
-    Atom atom_type;
-    char* name;
-    
-    attrs.override_redirect = True;
-    attrs.event_mask = PropertyChangeMask | StructureNotifyMask;
-    
-    xwindow =
-      XCreateWindow (xdisplay,
-                     RootWindow (xdisplay, 0),
-                     -100, -100, 1, 1,
-                     0,
-                     CopyFromParent,
-                     CopyFromParent,
-                     CopyFromParent,
-                     CWOverrideRedirect | CWEventMask,
-                     &attrs);
-    
-    atom_name = XInternAtom (xdisplay, "WM_NAME", TRUE);
-    g_assert (atom_name != None);
-    atom_type = XInternAtom (xdisplay, "STRING", TRUE);
-    g_assert (atom_type != None);
-    
-    name = "Fake Window";
-    XChangeProperty (xdisplay, 
-                     xwindow, atom_name,
-                     atom_type,
-                     8, PropModeReplace, (unsigned char *)name, strlen (name));
-  }
-  
-  XWindowEvent (xdisplay,
-                xwindow,
-                PropertyChangeMask,
-                &event);
-  
-  XDestroyWindow(xdisplay, xwindow);
-  
-  return event.xproperty.time;
-}
-#endif
-
-static void
-show_error_message (GError **error)
-{
-  GtkWidget *dialog;
-
-  /* FIXME better texts!!! */
-  dialog = gtk_message_dialog_new (NULL,
-                                   GTK_DIALOG_MODAL,
-                                   GTK_MESSAGE_ERROR,
-                                   GTK_BUTTONS_CLOSE,
-                                   _("Could not start Web"));
-  gtk_message_dialog_format_secondary_text
-    (GTK_MESSAGE_DIALOG (dialog),
-     _("Startup failed because of the following error:\n%s"),
-     (*error)->message);
-
-  g_clear_error (error);
-
-  gtk_dialog_run (GTK_DIALOG (dialog));
-}
-
 static EphyStartupFlags
 get_startup_flags (void)
 {
@@ -223,7 +182,7 @@ get_startup_flags (void)
 }
 
 int
-main (int argc,
+main (int   argc,
       char *argv[])
 {
   GOptionContext *option_context;
@@ -234,9 +193,13 @@ main (int argc,
   EphyShellStartupContext *ctx;
   EphyStartupFlags startup_flags;
   EphyEmbedShellMode mode;
-  EphyShell *ephy_shell;
   int status;
   EphyFileHelpersFlags flags;
+  GDesktopAppInfo *desktop_info = NULL;
+
+#if DEVELOPER_MODE
+  g_setenv ("GSETTINGS_SCHEMA_DIR", BUILD_ROOT "/data", FALSE);
+#endif
 
   /* Initialize the i18n stuff */
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
@@ -248,22 +211,22 @@ main (int argc,
    */
   LIBXML_TEST_VERSION;
 
-  notify_init (PACKAGE);
+  notify_init ("epiphany");
 
   /* If we're given -remote arguments, translate them */
   if (argc >= 2 && strcmp (argv[1], "-remote") == 0) {
     const char *opening, *closing;
     char *command, *argument;
-    char **arguments;
-    
+    char **arg_list;
+
     if (argc != 3) {
       g_print ("-remote allows exactly one argument\n");
       exit (1);
     }
-    
+
     opening = strchr (argv[2], '(');
     closing = strchr (argv[2], ')');
-    
+
     if (opening == NULL ||
         closing == NULL ||
         opening == argv[2] ||
@@ -271,9 +234,9 @@ main (int argc,
       g_print ("Invalid argument for -remote\n");
       exit (1);
     }
-    
+
     command = g_strstrip (g_strndup (argv[2], opening - argv[2]));
-    
+
     /* See http://lxr.mozilla.org/seamonkey/source/xpfe/components/xremote/src/XRemoteService.cpp
      * for the commands that mozilla supports; we'll just support openURL here.
      */
@@ -282,28 +245,28 @@ main (int argc,
       g_free (command);
       exit (1);
     }
-    
+
     g_free (command);
-    
+
     argument = g_strstrip (g_strndup (opening + 1, closing - opening - 1));
-    arguments = g_strsplit (argument, ",", -1);
+    arg_list = g_strsplit (argument, ",", -1);
     g_free (argument);
-    if (arguments == NULL) {
+    if (arg_list == NULL) {
       g_print ("Invalid argument for -remote\n");
-      
+
       exit (1);
     }
-    
+
     /* replace arguments */
-    argv[1] = g_strstrip (g_strdup (arguments[0]));
+    argv[1] = g_strstrip (g_strdup (arg_list[0]));
     argc = 2;
-    
-    g_strfreev (arguments);
+
+    g_strfreev (arg_list);
   }
-  
+
   /* Initialise our debug helpers */
   ephy_debug_init ();
-  
+
   /* get this early, since gdk will unset the env var */
   user_time = get_startup_id ();
 
@@ -327,7 +290,7 @@ main (int argc,
     g_option_context_free (option_context);
     exit (1);
   }
-        
+
   g_option_context_free (option_context);
 
   /* Some argument sanity checks*/
@@ -341,21 +304,35 @@ main (int argc,
     exit (1);
   }
 
-  if (application_mode && profile_directory == NULL) {
-    g_print ("--profile must be used when --application-mode is requested\n");
+  if (application_mode && profile_directory && !g_file_test (profile_directory, G_FILE_TEST_IS_DIR)) {
+    g_print ("--profile must be an existing directory when --application-mode is requested\n");
     exit (1);
   }
 
-  if (application_mode && !g_file_test (profile_directory, G_FILE_TEST_IS_DIR)) {
-      g_print ("--profile must be an existing directory when --application-mode is requested\n");
-      exit (1);
+  if (application_mode && !profile_directory) {
+    if (desktop_file_basename) {
+      desktop_info = g_desktop_app_info_new (desktop_file_basename);
+
+      if (desktop_info)
+        profile_directory = ephy_web_application_ensure_for_app_info (G_APP_INFO (desktop_info));
+
+      if (!profile_directory) {
+        g_print ("Invalid desktop file passed to --application-mode\n");
+        exit (1);
+      }
+    }
+  }
+
+  if (application_mode && profile_directory == NULL) {
+    g_print ("--profile must be used when --application-mode is requested without desktop file path\n");
+    exit (1);
   }
 
   if (incognito_mode && profile_directory == NULL)
-    profile_directory = g_strdup (ephy_dot_dir ());
+    profile_directory = ephy_default_dot_dir ();
 
   /* Start our services */
-  flags = EPHY_FILE_HELPERS_ENSURE_EXISTS;
+  flags = !application_mode ? EPHY_FILE_HELPERS_ENSURE_EXISTS : 0;
 
   if (incognito_mode || private_instance || application_mode)
     flags |= EPHY_FILE_HELPERS_PRIVATE_PROFILE;
@@ -364,10 +341,8 @@ main (int argc,
   if (profile_directory && !incognito_mode)
     flags |= EPHY_FILE_HELPERS_KEEP_DIR;
 
-  if (!ephy_file_helpers_init (profile_directory, flags,
-                               &error)) {
-    show_error_message (&error);
-    exit (1);
+  if (!ephy_file_helpers_init (profile_directory, flags, &error)) {
+    g_error ("Fatal initialization error: %s", error->message);
   }
 
   /* Run the migration in all cases, except when running a private
@@ -376,7 +351,7 @@ main (int argc,
   if (!(private_instance && profile_directory == NULL) && incognito_mode == FALSE) {
     /* If the migration fails we don't really want to continue. */
     if (!ephy_profile_utils_do_migration ((const char *)profile_directory, -1, FALSE)) {
-      g_print ("Failed to run the migrator process, Web will now abort.");
+      g_print ("Failed to run the migrator process, Web will now abort.\n");
       exit (1);
     }
   }
@@ -403,17 +378,6 @@ main (int argc,
     arguments = args;
   }
 
-#ifdef GDK_WINDOWING_X11
-  /* Get a timestamp manually if need be */
-  if (user_time == 0) {
-    GdkDisplay* display =
-      gdk_display_manager_get_default_display (gdk_display_manager_get ());
-    if (GDK_IS_X11_DISPLAY (display))
-      user_time =
-        slowly_and_stupidly_obtain_timestamp (GDK_DISPLAY_XDISPLAY (display));
-  }
-#endif
-
   /* Delete the requested web application, if any. Must happen after
    * ephy_file_helpers_init (). */
   if (application_to_delete) {
@@ -432,29 +396,14 @@ main (int argc,
   } else if (incognito_mode) {
     mode = EPHY_EMBED_SHELL_MODE_INCOGNITO;
   } else if (application_mode) {
-    char *app_name;
-    char *app_icon;
-
     mode = EPHY_EMBED_SHELL_MODE_APPLICATION;
 
-    app_name = g_strrstr (profile_directory, EPHY_WEB_APP_PREFIX);
-    app_icon = g_build_filename (profile_directory, EPHY_WEB_APP_ICON_NAME, NULL);
-
-    if (app_name) {
-      /* Skip the 'app-' part */
-      app_name += strlen (EPHY_WEB_APP_PREFIX);
-
-      g_set_prgname (app_name);
-      g_set_application_name (app_name);
-
-      gtk_window_set_default_icon_from_file (app_icon, NULL);
-
-      /* We need to re-set this because we have already parsed the
-       * options, which inits GTK+ and sets this as a side effect. */
-      gdk_set_program_class (app_name);
+    if (desktop_info) {
+      ephy_web_application_setup_from_desktop_file (desktop_info);
+      g_object_unref (desktop_info);
+    } else {
+      ephy_web_application_setup_from_profile_directory (profile_directory);
     }
-
-    g_free (app_icon);
   } else if (profile_directory) {
     /* This mode exists purely for letting EphyShell know it should
      * not consider this instance part of the unique application
@@ -467,7 +416,7 @@ main (int argc,
     g_set_prgname ("epiphany");
     g_set_application_name (_("Web"));
 
-    gtk_window_set_default_icon_name ("web-browser");
+    gtk_window_set_default_icon_name ("org.gnome.Epiphany");
   }
 
   _ephy_shell_create_instance (mode);
@@ -481,19 +430,27 @@ main (int argc,
   g_strfreev (arguments);
   ephy_shell = ephy_shell_get_default ();
   ephy_shell_set_startup_context (ephy_shell, ctx);
+
+  g_unix_signal_add (SIGINT, (GSourceFunc)handle_shutdown_signal, GINT_TO_POINTER (SIGINT));
+  g_unix_signal_add (SIGTERM, (GSourceFunc)handle_shutdown_signal, GINT_TO_POINTER (SIGTERM));
+
   status = g_application_run (G_APPLICATION (ephy_shell), argc, argv);
 
   /* Shutdown */
+  /* FIXME: should free this stuff even when exiting early */
   g_object_unref (ephy_shell);
+  g_free (desktop_file_basename);
   g_free (profile_directory);
 
   if (notify_is_initted ())
     notify_uninit ();
 
-  ephy_initial_state_save ();
   ephy_settings_shutdown ();
   ephy_file_helpers_shutdown ();
   xmlCleanupParser ();
 
+  if (shutdown_signum != 0)
+    raise (shutdown_signum);
+
   return status;
 }
