@@ -40,6 +40,7 @@
 #include "ephy-file-chooser.h"
 #include "ephy-file-helpers.h"
 #include "ephy-find-toolbar.h"
+#include "ephy-flatpak-utils.h"
 #include "ephy-gui.h"
 #include "ephy-header-bar.h"
 #include "ephy-history-dialog.h"
@@ -51,6 +52,7 @@
 #include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-string.h"
+#include "ephy-vcs-version.h"
 #include "ephy-web-app-utils.h"
 #include "ephy-zoom.h"
 
@@ -107,7 +109,7 @@ combo_box_changed_cb (GtkComboBox *combo_box,
 
   active = gtk_combo_box_get_active (combo_box);
   if (active == 0)
-    gtk_button_set_label (button, _("Ch_oose File"));
+    gtk_button_set_label (button, _("Ch_oose File…"));
   else if (active == 1)
     gtk_button_set_label (button, _("I_mport"));
 }
@@ -159,18 +161,22 @@ get_firefox_profiles (void)
   channel = g_io_channel_new_file (filename, "r", NULL);
   g_free (filename);
 
-  do {
-    g_io_channel_read_line (channel, &line, &length, NULL, NULL);
+  if (channel) {
+    do {
+      g_io_channel_read_line (channel, &line, &length, NULL, NULL);
 
-    profile = g_strdup_printf ("[Profile%d]\n", count);
-    if (g_strcmp0 (line, profile) == 0) {
-      profiles = g_slist_append (profiles, get_path (channel));
+      profile = g_strdup_printf ("[Profile%d]\n", count);
+      if (g_strcmp0 (line, profile) == 0) {
+        profiles = g_slist_append (profiles, get_path (channel));
 
-      count++;
-    }
-    g_free (profile);
-    g_free (line);
-  } while (length != 0);
+        count++;
+      }
+      g_free (profile);
+      g_free (line);
+    } while (length != 0);
+
+    g_io_channel_unref (channel);
+  }
 
   return profiles;
 }
@@ -558,8 +564,8 @@ window_cmd_show_about (GSimpleAction *action,
   GKeyFile *key_file;
   GBytes *bytes;
   GError *error = NULL;
-  char **list, **authors, **contributors, **past_authors, **artists, **documenters;
-  gsize n_authors, n_contributors, n_past_authors, n_artists, n_documenters, i, j;
+  char **list, **authors, **maintainers, **past_maintainers, **contributors, **artists, **documenters;
+  gsize n_authors, n_maintainers, n_past_maintainers, n_contributors, n_artists, n_documenters, i, j;
 
   key_file = g_key_file_new ();
   bytes = g_resources_lookup_data ("/org/gnome/epiphany/about.ini", 0, NULL);
@@ -570,12 +576,14 @@ window_cmd_show_about (GSimpleAction *action,
   }
   g_bytes_unref (bytes);
 
-  list = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Authors",
+  list = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Author",
                                      &n_authors, NULL);
+  maintainers = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Maintainers",
+                                            &n_maintainers, NULL);
+  past_maintainers = g_key_file_get_string_list (key_file, ABOUT_GROUP, "PastMaintainers",
+                                                 &n_past_maintainers, NULL);
   contributors = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Contributors",
                                              &n_contributors, NULL);
-  past_authors = g_key_file_get_string_list (key_file, ABOUT_GROUP, "PastAuthors",
-                                             &n_past_authors, NULL);
 
 #define APPEND(_to, _from) \
   _to[i++] = g_strdup (_from);
@@ -591,19 +599,23 @@ window_cmd_show_about (GSimpleAction *action,
   }
 
   authors = g_new (char *, (list ? n_authors : 0) +
-                   (contributors ? n_contributors : 0) +
-                   (past_authors ? n_past_authors : 0) + 7 + 1);
+                   (maintainers ? n_maintainers : 0) +
+                   (past_maintainers ? n_past_maintainers : 0) +
+                   (contributors ? n_contributors : 0) + 9 + 1);
   i = 0;
   APPEND_STRV_AND_FREE (authors, list);
   APPEND (authors, "");
+  APPEND (authors, _("Current maintainers:"));
+  APPEND_STRV_AND_FREE (authors, maintainers);
+  APPEND (authors, "");
   APPEND (authors, _("Contact us at:"));
   APPEND (authors, "<epiphany-list@gnome.org>");
   APPEND (authors, "");
+  APPEND (authors, _("Past maintainers:"));
+  APPEND_STRV_AND_FREE (authors, past_maintainers);
+  APPEND (authors, "");
   APPEND (authors, _("Contributors:"));
   APPEND_STRV_AND_FREE (authors, contributors);
-  APPEND (authors, "");
-  APPEND (authors, _("Past developers:"));
-  APPEND_STRV_AND_FREE (authors, past_authors);
   authors[i++] = NULL;
 
   list = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Artists", &n_artists, NULL);
@@ -635,10 +647,14 @@ window_cmd_show_about (GSimpleAction *action,
                               webkit_get_micro_version ());
 
   gtk_show_about_dialog (window ? GTK_WINDOW (window) : NULL,
+#if !TECH_PREVIEW
                          "program-name", _("Web"),
-                         "version", VERSION,
+#else
+                         "program-name", _("Epiphany Technology Preview"),
+#endif
+                         "version", VCSVERSION,
                          "copyright", "Copyright © 2002–2004 Marco Pesenti Gritti\n"
-                         "Copyright © 2003–2017 The Web Developers",
+                         "Copyright © 2003–2018 The Web Developers",
                          "artists", artists,
                          "authors", authors,
                          "comments", comments,
@@ -654,7 +670,11 @@ window_cmd_show_about (GSimpleAction *action,
                          "translator-credits", _("translator-credits"),
                          "logo-icon-name", "org.gnome.Epiphany",
                          "website", "https://wiki.gnome.org/Apps/Web",
+#if !TECH_PREVIEW
                          "website-label", _("Web Website"),
+#else
+                         "website-label", _("Website"),
+#endif
                          "license-type", GTK_LICENSE_GPL_3_0,
                          "wrap-license", TRUE,
                          NULL);
@@ -694,7 +714,7 @@ window_cmd_navigation (GSimpleAction *action,
   WebKitWebView *web_view;
 
   embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-  g_return_if_fail (embed != NULL);
+  g_assert (embed != NULL);
 
   web_view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
 
@@ -717,7 +737,7 @@ window_cmd_navigation_new_tab (GSimpleAction *action,
   WebKitWebView *web_view;
 
   embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-  g_return_if_fail (embed != NULL);
+  g_assert (embed != NULL);
 
   web_view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
 
@@ -770,7 +790,7 @@ window_cmd_stop (GSimpleAction *action,
 
   embed = ephy_embed_container_get_active_child
             (EPHY_EMBED_CONTAINER (window));
-  g_return_if_fail (embed != NULL);
+  g_assert (embed != NULL);
 
   gtk_widget_grab_focus (GTK_WIDGET (embed));
 
@@ -811,7 +831,7 @@ window_cmd_reload (GSimpleAction *action,
 
   embed = ephy_embed_container_get_active_child
             (EPHY_EMBED_CONTAINER (window));
-  g_return_if_fail (embed != NULL);
+  g_assert (embed != NULL);
 
   gtk_widget_grab_focus (GTK_WIDGET (embed));
 
@@ -844,6 +864,23 @@ void window_cmd_combined_stop_reload (GSimpleAction *action,
   g_variant_unref (state);
 }
 
+void
+window_cmd_page_menu (GSimpleAction *action,
+                      GVariant      *parameter,
+                      gpointer       user_data)
+
+{
+  EphyWindow *window = EPHY_WINDOW (user_data);
+  EphyHeaderBar *header_bar;
+  GtkMenuButton *button;
+  GtkPopover *popover;
+
+  header_bar = EPHY_HEADER_BAR (ephy_window_get_header_bar (window));
+  button = GTK_MENU_BUTTON (ephy_header_bar_get_page_menu_button (header_bar));
+  popover = gtk_menu_button_get_popover (button);
+  gtk_popover_popup (popover);
+}
+
 void
 window_cmd_new_tab (GSimpleAction *action,
                     GVariant      *parameter,
@@ -897,6 +934,28 @@ window_cmd_open (GSimpleAction *action,
   gtk_native_dialog_show (GTK_NATIVE_DIALOG (dialog));
 }
 
+void
+window_cmd_open_in_browser (GSimpleAction *action,
+                            GVariant      *parameter,
+                            gpointer       user_data)
+{
+  EphyWindow *window = user_data;
+  EphyEmbed *embed;
+  EphyWebView *view;
+  const char *uri;
+
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  g_assert (embed != NULL);
+
+  view = ephy_embed_get_web_view (embed);
+  uri = ephy_web_view_get_address (view);
+
+  ephy_file_open_uri_in_default_browser (uri, GDK_CURRENT_TIME,
+                                         gtk_window_get_screen (GTK_WINDOW (window)));
+
+  ephy_window_close_active_child (window);
+}
+
 typedef struct {
   EphyWebView *view;
   GtkWidget *image;
@@ -1340,8 +1399,11 @@ window_cmd_save_as_application (GSimpleAction *action,
   char *markup;
   char *escaped_address;
 
+  if (ephy_is_running_inside_flatpak ())
+    return;
+
   embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-  g_return_if_fail (embed != NULL);
+  g_assert (embed != NULL);
 
   view = EPHY_WEB_VIEW (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
 
@@ -1435,7 +1497,8 @@ get_suggested_filename (EphyEmbed *embed)
     suggested_filename = g_strdup (webkit_uri_response_get_suggested_filename (response));
     if (!suggested_filename) {
       SoupURI *soup_uri = soup_uri_new (webkit_web_resource_get_uri (web_resource));
-      suggested_filename = g_path_get_basename (soup_uri->path);
+      char *last_slash = strrchr (soup_uri->path, '/');
+      suggested_filename = soup_uri_decode (last_slash ? (last_slash + 1) : soup_uri->path);
       soup_uri_free (soup_uri);
     }
   }
@@ -1477,7 +1540,7 @@ window_cmd_save_as (GSimpleAction *action,
   char *suggested_filename;
 
   embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-  g_return_if_fail (embed != NULL);
+  g_assert (embed != NULL);
 
   dialog = ephy_create_file_chooser (_("Save"),
                                      GTK_WIDGET (window),
@@ -1556,7 +1619,7 @@ window_cmd_cut (GSimpleAction *action,
   } else {
     EphyEmbed *embed;
     embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-    g_return_if_fail (embed != NULL);
+    g_assert (embed != NULL);
 
     webkit_web_view_execute_editing_command (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed), WEBKIT_EDITING_COMMAND_CUT);
   }
@@ -1576,7 +1639,7 @@ window_cmd_copy (GSimpleAction *action,
     EphyEmbed *embed;
 
     embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-    g_return_if_fail (embed != NULL);
+    g_assert (embed != NULL);
 
     webkit_web_view_execute_editing_command (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed), WEBKIT_EDITING_COMMAND_COPY);
   }
@@ -1596,7 +1659,7 @@ window_cmd_paste (GSimpleAction *action,
     EphyEmbed *embed;
 
     embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-    g_return_if_fail (embed != NULL);
+    g_assert (embed != NULL);
 
     webkit_web_view_execute_editing_command (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed), WEBKIT_EDITING_COMMAND_PASTE);
   }
@@ -1616,7 +1679,7 @@ window_cmd_delete (GSimpleAction *action,
     EphyEmbed *embed;
 
     embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-    g_return_if_fail (embed != NULL);
+    g_assert (embed != NULL);
 
     /* FIXME: TODO */
 #if 0
@@ -1637,7 +1700,7 @@ window_cmd_print (GSimpleAction *action,
 
   embed = ephy_embed_container_get_active_child
             (EPHY_EMBED_CONTAINER (window));
-  g_return_if_fail (EPHY_IS_EMBED (embed));
+  g_assert (EPHY_IS_EMBED (embed));
   view = ephy_embed_get_web_view (embed);
 
   ephy_web_view_print (view);
@@ -1773,6 +1836,55 @@ view_source_embedded (const char *uri, EphyEmbed *embed)
   gtk_widget_grab_focus (GTK_WIDGET (new_embed));
 }
 
+static void
+tmp_source_delete_cb (GObject *source, GAsyncResult *result, gpointer data)
+{
+  GFile *file = G_FILE (source);
+  GError *error = NULL;
+
+  if (!g_file_delete_finish (G_FILE (source), result, &error)) {
+    g_warning ("Failed to delete %s: %s", g_file_get_path (file), error->message);
+    g_error_free (error);
+  }
+
+  g_object_unref (file);
+}
+
+static void
+delete_tmp_source_file (GOutputStream *ostream)
+{
+  char *uri;
+  GFile *file;
+
+  g_object_get (G_OBJECT (ostream), "ephy-save-temp-source-uri", &uri, NULL);
+  file = g_file_new_for_uri (uri);
+  g_file_delete_async (file, G_PRIORITY_LOW, NULL, tmp_source_delete_cb, NULL);
+
+  g_free (uri);
+}
+
+static void
+portal_opened_cb (GObject *source, GAsyncResult *result, gpointer data)
+{
+  char *path = data;
+  GFile *file;
+  gboolean ret;
+  GError *error = NULL;
+
+  ret = ephy_open_file_via_flatpak_portal_finish (result, &error);
+  if (!ret) {
+    /* It can be canceled by the user, even without a cancellable. */
+    if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+      g_warning ("Failed to open file via flatpak portal: %s", error->message);
+    g_error_free (error);
+  }
+
+  file = g_file_new_for_path (path);
+  g_free (path);
+
+  g_file_delete_async (file, G_PRIORITY_LOW, NULL, tmp_source_delete_cb, NULL);
+}
+
 static void
 save_temp_source_close_cb (GOutputStream *ostream, GAsyncResult *result, gpointer data)
 {
@@ -1784,6 +1896,7 @@ save_temp_source_close_cb (GOutputStream *ostream, GAsyncResult *result, gpointe
   if (error) {
     g_warning ("Unable to close file: %s", error->message);
     g_error_free (error);
+    delete_tmp_source_file (ostream);
     return;
   }
 
@@ -1791,6 +1904,14 @@ save_temp_source_close_cb (GOutputStream *ostream, GAsyncResult *result, gpointe
 
   file = g_file_new_for_uri (uri);
 
+  if (ephy_is_running_inside_flatpak ()) {
+    const char *path;
+
+    path = g_file_get_path (file);
+    ephy_open_file_via_flatpak_portal (path, NULL, portal_opened_cb, g_strdup (path));
+    goto out;
+  }
+
   if (!ephy_file_launch_handler ("text/plain", file, gtk_get_current_event_time ())) {
     /* Fallback to view the source inside the browser */
     EphyEmbed *embed;
@@ -1801,8 +1922,9 @@ save_temp_source_close_cb (GOutputStream *ostream, GAsyncResult *result, gpointe
                                             "ephy-save-temp-source-embed");
     view_source_embedded (uri, embed);
   }
-  g_object_unref (ostream);
 
+out:
+  g_object_unref (ostream);
   g_object_unref (file);
 }
 
@@ -1857,6 +1979,7 @@ get_main_resource_data_cb (WebKitWebResource *resource, GAsyncResult *result, GO
   if (error) {
     g_warning ("Unable to get main resource data: %s", error->message);
     g_error_free (error);
+    delete_tmp_source_file (ostream);
     return;
   }
 
@@ -1916,19 +2039,26 @@ save_temp_source (EphyEmbed *embed,
 {
   GFile *file;
   char *tmp, *base;
-  const char *static_temp_dir;
 
-  static_temp_dir = ephy_file_tmp_dir ();
-  if (static_temp_dir == NULL) {
-    return;
+  if (ephy_is_running_inside_flatpak ()) {
+    /* It has to go here because the portal has no access to our tmpfs.
+     * This means we have to delete it manually! */
+    base = g_build_filename (g_get_user_cache_dir (), "tmp", "viewsourceXXXXXX", NULL);
+  } else {
+    const char *static_temp_dir;
+
+    static_temp_dir = ephy_file_tmp_dir ();
+    if (static_temp_dir == NULL)
+      return;
+
+    base = g_build_filename (static_temp_dir, "viewsourceXXXXXX", NULL);
   }
 
-  base = g_build_filename (static_temp_dir, "viewsourceXXXXXX", NULL);
   tmp = ephy_file_tmp_filename (base, "html");
   g_free (base);
-  if (tmp == NULL) {
+
+  if (tmp == NULL)
     return;
-  }
 
   file = g_file_new_for_path (tmp);
   g_file_replace_async (file, NULL, FALSE,
@@ -1953,7 +2083,7 @@ window_cmd_page_source (GSimpleAction *action,
 
   embed = ephy_embed_container_get_active_child
             (EPHY_EMBED_CONTAINER (window));
-  g_return_if_fail (embed != NULL);
+  g_assert (embed != NULL);
 
   address = ephy_web_view_get_address (ephy_embed_get_web_view (embed));
 
@@ -1993,7 +2123,7 @@ window_cmd_toggle_inspector (GSimpleAction *action,
 
   embed = ephy_embed_container_get_active_child
             (EPHY_EMBED_CONTAINER (window));
-  g_return_if_fail (embed != NULL);
+  g_assert (embed != NULL);
 
   gtk_widget_grab_focus (GTK_WIDGET (embed));
 
@@ -2023,7 +2153,7 @@ window_cmd_select_all (GSimpleAction *action,
 
     embed = ephy_embed_container_get_active_child
               (EPHY_EMBED_CONTAINER (window));
-    g_return_if_fail (embed != NULL);
+    g_assert (embed != NULL);
 
     webkit_web_view_execute_editing_command (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed), "SelectAll");
   }
@@ -2042,7 +2172,7 @@ window_cmd_send_to (GSimpleAction *action,
 
   embed = ephy_embed_container_get_active_child
             (EPHY_EMBED_CONTAINER (window));
-  g_return_if_fail (embed != NULL);
+  g_assert (embed != NULL);
 
   location = ephy_web_view_get_address (ephy_embed_get_web_view (embed));
   title = ephy_embed_get_title (embed);
@@ -2148,7 +2278,7 @@ window_cmd_tabs_previous (GSimpleAction *action,
   GtkWidget *nb;
 
   nb = ephy_window_get_notebook (EPHY_WINDOW (user_data));
-  g_return_if_fail (nb != NULL);
+  g_assert (nb != NULL);
 
   ephy_notebook_prev_page (EPHY_NOTEBOOK (nb));
 }
@@ -2161,7 +2291,7 @@ window_cmd_tabs_next (GSimpleAction *action,
   GtkWidget *nb;
 
   nb = ephy_window_get_notebook (EPHY_WINDOW (user_data));
-  g_return_if_fail (nb != NULL);
+  g_assert (nb != NULL);
 
   ephy_notebook_next_page (EPHY_NOTEBOOK (nb));
 }
@@ -2278,7 +2408,7 @@ window_cmd_tabs_close (GSimpleAction *action,
   }
 
   embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-  g_return_if_fail (embed != NULL);
+  g_assert (embed != NULL);
 
   g_signal_emit_by_name (notebook, "tab-close-request", embed);
 }
