@@ -1,35 +1,30 @@
-/* vim: set sw=2 ts=2 sts=2 et: */
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /*
- * ephy-web-view-test.c
- * This file is part of Epiphany
+ *  Copyright © 2012 Igalia S.L.
  *
- * Copyright © 2012 - Igalia S.L.
+ *  This file is part of Epiphany.
  *
- * Epiphany is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
  *
- * Epiphany is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ *  Epiphany is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with Epiphany; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA  02110-1301  USA
+ *  You should have received a copy of the GNU General Public License
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 #include "ephy-debug.h"
 #include "ephy-embed-prefs.h"
-#include "ephy-embed-private.h"
 #include "ephy-embed-utils.h"
 #include "ephy-file-helpers.h"
 #include "ephy-history-service.h"
-#include "ephy-private.h"
+#include "ephy-search-engine-manager.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-web-view.h"
@@ -44,16 +39,16 @@
 #define SERVER_PORT 12321
 
 static void
-server_callback (SoupServer *server,
-                 SoupMessage *msg,
-                 const char *path,
-                 GHashTable *query,
+server_callback (SoupServer        *server,
+                 SoupMessage       *msg,
+                 const char        *path,
+                 GHashTable        *query,
                  SoupClientContext *context,
-                 gpointer data)
+                 gpointer           data)
 {
-  if (g_str_equal (path, "/cancelled"))
+  if (!strcmp (path, "/cancelled"))
     soup_message_set_status (msg, SOUP_STATUS_CANT_CONNECT);
-  else if (g_str_equal (path, "/redirect")) {
+  else if (!strcmp (path, "/redirect")) {
     soup_message_set_status (msg, SOUP_STATUS_MOVED_PERMANENTLY);
     soup_message_headers_append (msg->response_headers, "Location", "/redirect-result");
   } else
@@ -139,7 +134,7 @@ static const URLTest test_load_url[] = {
 static void
 test_ephy_web_view_load_url (void)
 {
-  int i;
+  guint i;
 
   for (i = 0; i < G_N_ELEMENTS (test_load_url); i++) {
     URLTest test;
@@ -209,7 +204,7 @@ test_ephy_web_view_non_search_regex (void)
 {
   GRegex *regex_non_search, *regex_domain;
   GError *error = NULL;
-  int i;
+  guint i;
 
   regex_non_search = g_regex_new (EPHY_WEB_VIEW_NON_SEARCH_REGEX,
                                   0, G_REGEX_MATCH_NOTEMPTY, &error);
@@ -247,8 +242,8 @@ test_ephy_web_view_non_search_regex (void)
 }
 
 typedef struct {
-  char *url;
-  char *expected;
+  const char *url;
+  const char *expected;
 } normalize_or_autosearch_t;
 
 normalize_or_autosearch_t normalize_or_autosearch_test_ddg[] = {
@@ -273,14 +268,15 @@ normalize_or_autosearch_t normalize_or_autosearch_test_google[] = {
 };
 
 static void
-verify_normalize_or_autosearch_urls (EphyWebView *view,
+verify_normalize_or_autosearch_urls (EphyWebView               *view,
                                      normalize_or_autosearch_t *test,
-                                     gint n_tests)
+                                     gint                       n_tests)
 {
   int i;
 
   for (i = 0; i < n_tests; i++) {
-    char *url, *result;
+    const char *url;
+    char *result;
 
     url = test[i].url;
 
@@ -294,30 +290,37 @@ verify_normalize_or_autosearch_urls (EphyWebView *view,
 static void
 test_ephy_web_view_normalize_or_autosearch (void)
 {
-  char *default_engine_url;
+  char *default_engine;
   EphyWebView *view;
-  
+  EphySearchEngineManager *manager;
+  EphyEmbedShell *shell;
+
+
   view = EPHY_WEB_VIEW (ephy_web_view_new ());
-  default_engine_url = g_settings_get_string (EPHY_SETTINGS_MAIN,
-                                              EPHY_PREFS_KEYWORD_SEARCH_URL);
 
-  g_settings_set_string (EPHY_SETTINGS_MAIN,
-                         EPHY_PREFS_KEYWORD_SEARCH_URL,
-                         "http://duckduckgo.com/?q=%s&t=epiphany");
+  shell = ephy_embed_shell_get_default ();
+  manager = ephy_embed_shell_get_search_engine_manager (shell);
 
+
+  default_engine = ephy_search_engine_manager_get_default_engine (manager);
+  ephy_search_engine_manager_add_engine (manager,
+                                         "org.gnome.Epiphany.EphyWebViewTest",
+                                         "http://duckduckgo.com/?q=%s&t=epiphany",
+                                         "");
+  g_assert (ephy_search_engine_manager_set_default_engine (manager, "org.gnome.Epiphany.EphyWebViewTest"));
   verify_normalize_or_autosearch_urls (view, normalize_or_autosearch_test_ddg, G_N_ELEMENTS (normalize_or_autosearch_test_ddg));
 
-  g_settings_set_string (EPHY_SETTINGS_MAIN,
-                         EPHY_PREFS_KEYWORD_SEARCH_URL,
-                         "http://www.google.com/?q=%s");
+  ephy_search_engine_manager_modify_engine (manager,
+                                            "org.gnome.Epiphany.EphyWebViewTest",
+                                            "http://www.google.com/?q=%s",
+                                            "");
 
+  g_assert (ephy_search_engine_manager_set_default_engine (manager, default_engine));
   verify_normalize_or_autosearch_urls (view, normalize_or_autosearch_test_google, G_N_ELEMENTS (normalize_or_autosearch_test_google));
 
-  g_settings_set_string (EPHY_SETTINGS_MAIN,
-                         EPHY_PREFS_KEYWORD_SEARCH_URL,
-                         default_engine_url);
+  ephy_search_engine_manager_delete_engine (manager, "org.gnome.Epiphany.EphyWebViewTest");
 
-  g_free (default_engine_url);
+  g_free (default_engine);
   g_object_unref (g_object_ref_sink (view));
 }
 
@@ -334,10 +337,10 @@ quit_main_loop_when_load_finished (WebKitWebView *view, WebKitLoadEvent load_eve
 static guint back_forward_list_counter = 0;
 
 static void
-back_forward_list_changed (WebKitBackForwardList *list,
+back_forward_list_changed (WebKitBackForwardList     *list,
                            WebKitBackForwardListItem *added_item,
-                           GList *removed_items,
-                           GMainLoop *loop)
+                           GList                     *removed_items,
+                           GMainLoop                 *loop)
 {
   back_forward_list_counter--;
 
@@ -347,13 +350,13 @@ back_forward_list_changed (WebKitBackForwardList *list,
 
 static void
 wait_until_back_forward_list_changes (WebKitWebView *view,
-                                      GMainLoop *loop)
+                                      GMainLoop     *loop)
 {
-    WebKitBackForwardList *back_forward_list = webkit_web_view_get_back_forward_list (view);
-    g_signal_connect (back_forward_list, "changed", G_CALLBACK (back_forward_list_changed), loop);
+  WebKitBackForwardList *back_forward_list = webkit_web_view_get_back_forward_list (view);
+  g_signal_connect (back_forward_list, "changed", G_CALLBACK (back_forward_list_changed), loop);
 }
 
-static GMainLoop*
+static GMainLoop *
 setup_ensure_back_forward_list_changes (EphyWebView *view)
 {
   GMainLoop *loop;
@@ -379,70 +382,67 @@ ensure_back_forward_list_changes (GMainLoop *loop)
 static void
 test_ephy_web_view_provisional_load_failure_updates_back_forward_list (void)
 {
-    GMainLoop *loop;
-    EphyWebView *view;
-    const char *bad_url;
+  GMainLoop *loop;
+  EphyWebView *view;
+  const char *bad_url;
 
-    view = EPHY_WEB_VIEW (ephy_web_view_new ());
+  view = EPHY_WEB_VIEW (ephy_web_view_new ());
 
-    loop = setup_ensure_back_forward_list_changes (view);
-    bad_url = "http://localhost:2984375932/";
+  loop = setup_ensure_back_forward_list_changes (view);
+  bad_url = "http://localhost:2984375932/";
 
-    ephy_web_view_load_url (view, bad_url);
+  ephy_web_view_load_url (view, bad_url);
 
-    ensure_back_forward_list_changes (loop);
+  ensure_back_forward_list_changes (loop);
 
-    g_assert (webkit_back_forward_list_get_current_item (
-      webkit_web_view_get_back_forward_list (WEBKIT_WEB_VIEW (view))));
+  g_assert (webkit_back_forward_list_get_current_item (
+              webkit_web_view_get_back_forward_list (WEBKIT_WEB_VIEW (view))));
 
-    g_assert_cmpstr (bad_url, ==, webkit_back_forward_list_item_get_uri (
-      webkit_back_forward_list_get_current_item (
-        webkit_web_view_get_back_forward_list (WEBKIT_WEB_VIEW (view)))));
+  g_assert_cmpstr (bad_url, ==, webkit_back_forward_list_item_get_uri (
+                     webkit_back_forward_list_get_current_item (
+                       webkit_web_view_get_back_forward_list (WEBKIT_WEB_VIEW (view)))));
 
-    g_object_unref (g_object_ref_sink (view));
+  g_object_unref (g_object_ref_sink (view));
 }
 
-static gboolean
+static void
 visit_url_cb (EphyHistoryService *service,
-              const char *url,
-              EphyHistoryPageVisit visit_type,
-              gpointer user_data)
+              EphyHistoryURL     *url,
+              gpointer            user_data)
 {
-    /* We are only loading an error page, this code should never be
-     * reached. */
-    g_assert_not_reached ();
-
-    return FALSE;
+  /* We are only loading an error page, this code should never be
+   * reached. */
+  g_assert_not_reached ();
 }
 
 static void
 test_ephy_web_view_error_pages_not_stored_in_history (void)
 {
-    GMainLoop *loop;
-    EphyWebView *view;
-    const char *bad_url;
-    EphyHistoryService *history_service;
-    EphyEmbedShell *embed_shell = ephy_embed_shell_get_default ();
+  GMainLoop *loop;
+  EphyWebView *view;
+  const char *bad_url;
+  EphyHistoryService *history_service;
+  EphyEmbedShell *embed_shell = ephy_embed_shell_get_default ();
 
-    view = EPHY_WEB_VIEW (ephy_web_view_new ());
-    loop = g_main_loop_new (NULL, FALSE);
-    bad_url = "http://localhost:2984375932/";
+  view = EPHY_WEB_VIEW (ephy_web_view_new ());
+  loop = g_main_loop_new (NULL, FALSE);
+  bad_url = "http://localhost:2984375932/";
 
-    history_service = EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (embed_shell));
-    g_assert (history_service);
-    g_signal_connect (history_service, "visit-url",
-                      G_CALLBACK (visit_url_cb), NULL);
-    
-    ephy_web_view_load_url (view, bad_url);
+  history_service = ephy_embed_shell_get_global_history_service (embed_shell);
+  g_assert (history_service);
+  g_signal_connect (history_service, "visit-url",
+                    G_CALLBACK (visit_url_cb), NULL);
 
-    g_signal_connect (view, "load-changed",
-                      G_CALLBACK (quit_main_loop_when_load_finished), loop);
+  ephy_web_view_load_url (view, bad_url);
 
-    g_main_loop_run (loop);
-    g_main_loop_unref (loop);
-    g_signal_handlers_disconnect_by_func (history_service, G_CALLBACK (visit_url_cb), NULL);
+  g_signal_connect (view, "load-changed",
+                    G_CALLBACK (quit_main_loop_when_load_finished), loop);
 
-    g_object_unref (g_object_ref_sink (view));
+  g_main_loop_run (loop);
+  g_main_loop_unref (loop);
+  g_signal_handlers_disconnect_by_func (history_service, G_CALLBACK (visit_url_cb), NULL);
+
+  g_object_unref (g_object_ref_sink (view));
 }
 
 int
@@ -467,7 +467,7 @@ main (int argc, char *argv[])
   server = soup_server_new (NULL, NULL);
   soup_server_add_handler (server, NULL, server_callback, NULL, NULL);
   soup_server_listen_local (server, SERVER_PORT,
-			    SOUP_SERVER_LISTEN_IPV4_ONLY, NULL);
+                            SOUP_SERVER_LISTEN_IPV4_ONLY, NULL);
 
   g_test_add_func ("/embed/ephy-web-view/non_search_regex",
                    test_ephy_web_view_non_search_regex);
